#!/usr/bin/perl

################################### SR.CGI #################################
#
# V2.1.7 Single User
#
# Perl front end to the C surfreport engine
# It gives an immediate notice of what is being done and launches the 
# surfreport engine in the background
# sr.cgi receives and examinins data from the FORM interface to SR2
# it checks the data for the configuration file and tries to open
# (or create with the correct permissions) the files sr2 will need
# All errors are gathered into an HTML page.
# If all goes well, the script will look for log files in the given folder
# and size them to give the user an idea of how long the report will take
# It will create an HTML page with a Client Pull meta tag to retrieve
# the report automatically when availble
# sr.cgi gathers all information needed by subsequent sr2 programs into
# a settings file. The file name created is unique and is passed along to
# all programs that need it  
# In order to launch the core engine in the background and regain control
# this script will fork after the settings have been written to file
# The parent process will simply print out the html page and exit
# the child process will close stdout, causing the HTML page to be flushed
# and launch srengine.cgi (c code) in the background
#
# Author:
#
# Michel G. Delory
# 
# created March 1996
# last rev: Dec 3 1997
#
# version 1 by 
# Richard McGillis
#
# copyright 1996,1997 BienLogic Inc
#
 
########################### ARRAYS & STRINGS ################################

$httpHeader = "Content-type: text/html\n\n";
$errorHeader = "<HTML><HEAD><TITLE>SurfReport Error</TITLE></HEAD>";
$textLine = "-----------------------\n\n";
$htmlLine = "<P>\n<HR WIDTH = 80%>\n";  
$formErr = "Form data:";
$confErr = "Configuration file:";
$sysErr = "System error:";

@DaysInMonth = ( 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
$htmlCopyright =    "<H5>SurfReport 2.1.7<br>Copyright &copy; 1996,1997 <A HREF = \"http://www.bienlogic.com\">BienLogic, Inc.</A></H5>";
$textCopyright =    "\nSurfReport 2.1.7\nCopyright (c) 1996,1997 BienLogic, Inc\n";
 
######################### ENVIRONMENT ##########################


if (!defined ($ENV{SURFREPORT_CONFIG}) || 
    !defined ($ENV{SURFREPORT_LIB}))
{
    #can't acces config file, forget it
    #can't access error routines in config hence this
    $err = "SurfReport environment is not available.\n";
    $err .= "Can't access your configuration file\n";
    $err .= "You must run this script through the SurfReport dispatcher\n";

    print $httpHeader;
    print $errorHeader;
    print "<body BGCOLOR = #FFFFFF TEXT = #OOOOOO >\n";
    print "<H3>Environment Error</H3>\n";
    
    print "<pre>\n\n";
    print $err;
    print "\n\n</pre>";

    print $htmlCopyright;
    

    print "</BODY></HTML>";
    exit (-1);

}

# read in configuration files
require $ENV{SURFREPORT_CONFIG};
require $ENV{SURFREPORT_LIB};

if ($ENV{STAND_ALONE} eq "YES")
{
	$ENV{DISPATCHER_URL} = $dispatcherURL; # from config file
}

#get the base path for all SR output files
#and make sure all needed env vars are available
if (!defined ($surfOut = $ENV{SURFREPORT_OUT}) ||
    !defined ($ENV{SURFREPORT_HOME}) ||
    !defined ($ENV{DISPATCHER_URL}) ||
    !defined ($platform = $ENV{PLATFORM}))
{
    # exit now if env vars not available
    $err = "SurfReport environment is not available.\n";
    $err .= "You must run this script through the SurfReport dispatcher\n";

    &Error($err);
    
    exit (-1);

}

if ($platform eq "OS_NT")
{
	$coreApp = "srengine.exe";
}
else
{
	$coreApp = "srengine.cgi";
}

$converterApp = "srreport.cgi";

######################### FORM PROCESSING & VARS INIT ######################

#make a unique name base for temporary files
#use the current time and the perl process id
$tmpName = time();
$tmpName .= $$;

$settingsFile = $surfOut . "tmp/";
$settingsFile .= $tmpName;

$waitSR = 3;

# when used with doform on the command line
if ($ENV{STAND_ALONE} eq "YES")
{
    $standAlone = 1;
    
    # v2.09 fetch form parameters from a file
    # on the command line
    &GetFormParamFile ($ARGV[0]);
}
else
{
	if (defined ($ENV{SURFREPORT_DIRECTOUTPUT}))
    {
        $standAlone = 1;
        $ENV{STAND_ALONE} = "YES";
    }
    else
    {
        $standAlone = 0;
    }
    
    $formData = &GetFormInput();  
}




# first check things that can go wrong
# and gather all errors

$errMsg = "";
$numErr = 0;
				 
#files matching this string will be processed
$matchLog = $FORM {MatchLog};

&FormatTextBox (*matchLog);
    
if ($matchLog eq '')
{
    $numErr++;
    $errMsg .= "$numErr - $formErr The filename field is empty.\n";
    
}
else
{
    $mLogCase = $FORM{MLogCase} eq 'on' ? 1 : 0;
    $mLogMatch = $FORM{MLogMatch}; # radio button to implement
} 

# check password				
$passwordOut = $FORM{Password};
if ($passwordOut ne $password)
{
    $numErr++;
    $errMsg .= "$numErr - $formErr The Password is incorrect.\n";
    
}

# series of file checks all with the same logic:
# 1-check base name in config file, reject blank names
# 2-try opening the file from the correct directory
# 3-if the file can be opened, just close it
# 4-if the file cannot be opened, try to create a new file
# 5-if file creation is not succesful, output an error message
# 6-change file mod on the newly created file otherwise
# 7-close the file
    
# check name base
#future hook, nameBase is available from the interface
#form, easy to set up a field to change it
#nameBase will be retained by all report links, including graphics
#which will make reports more moveable 
$nameBase = $FORM{NameBase};
if ( $nameBase eq "")
{
    $numErr++;
    $errMsg .= "$numErr - $formErr The name for SurfReport output files is blank.\n";
    
}
elsif ( $nameBase =~ /\W/)
{
    $numErr++;
    $errMsg .= "$numErr - $formErr The name for SurfReport output files contains non-word
characters.\n";
}
else
{
    # check data file

    # new names for output files in V2.07
    # all output files share the same base name
    # only the extension changes
   
    $dataFileName = $surfOut . $nameBase . ".txt";
    
    #try out data file, to be able to give an err message right away
    unless ( open (DATA, "$dataFileName"))
    {
	# create the file if does not exist yet
	if (!open (DATA, ">$dataFileName"))
	{
	    $numErr++;
	    $errMsg .= sprintf("$numErr - $sysErr  Can't create or open file '$dataFileName':\n\t%s\n",
$!);
	    
	}
	else
	{
	    chmod (0666, $dataFileName);
	}
    }
    close (DATA);


    # check internal log file
    $logFileName = $surfOut . $nameBase . ".log";
    
    #try out log file, to be able to give an err message right away
    unless ( open (LOG, "$logFileName"))
    {
	if (!open (LOG, ">$logFileName"))
	{
	    $numErr++;
	    $errMsg .= sprintf ("$numErr - $sysErr  Can't create or open file '$logFileName':\n\t%s\n",
$!);
		
	}
	else
	{
	    chmod (0666, $logFileName);
	}
    }
    close (LOG);

    # check data file's table of content
    $tocFileName = $surfOut . $nameBase . ".toc";;
    
    #try out toc file, to be able to give an err message right away
    unless (open (TOC, "$tocFileName"))
    {
	if (!open (TOC, ">$tocFileName"))
	{
	    $numErr++;
	    $errMsg .= sprintf ("$numErr - $sysErr  Can't create or open file '$tocFileName':\n\t%s\n",
$!);
		
	}
	else
	{
	    chmod (0666, $tocFileName);
	}
    }
    close (TOC);    

    # check the report html file
    $outputFileName = $surfOut . $nameBase . ".html";
    
    #try out output file, to be able to give an err message right away
    unless ( open (OUT, "$outputFileName"))
    {
	if (!open (OUT, ">$outputFileName"))
	{
	    $numErr++;
	    $errMsg .= sprintf ("$numErr - $sysErr  Can't create or open file
'$outputFileName':\n\t%s\n", $!);
		
	}
	else
	{
	    chmod (0666, $outputFileName);
	}
    }
    close (OUT);
}



# retrieve requested time period
if( $FORM{ Whichdate} eq 'Dodate1')
{
    # cover entire period available
    $startTime = 0 ;
    $endTime = 0x0FFFFFFF;
}
else
{
    $sy = $FORM{Startyear};
    $ey = $FORM{Endyear};
    $sm = $FORM{Startmonth};
    $em = $FORM{Endmonth};
    $sd = $FORM{Startday};
    $ed = $FORM{Endday};
	
    $fInvalidDate = 0;
    # check if dates in form are invalid ( like Feb 30 1996 )
    if ( ! &IsValidDate( $sy, $sm, $sd))
    {
	$numErr++;
	$errMsg .= "$numErr - $formErr Start date: $sm/$sd/$sy is invalid.\n";
	$fInvalidDate = 1;
    }
    if ( ! &IsValidDate( $ey, $em, $ed))
    {
	$numErr++;
	$errMsg .= "$numErr - $formErr End date: $em/$ed/$ey is invalid.\n";
	$fInvalidDate = 1;
    }
    
    if ($fInvalidDate == 0)
    {
	# transform dates into integer timestamps
	$startTime = &TimeDateToStamp( $sy, $sm, $sd);
	$endTime = &TimeDateToStamp( $ey, $em, $ed);
				 
	if ($endTime < $startTime)
	{
	    $numErr++;
	    $errMsg .= "$numErr - $formErr Ending date must not be earlier than starting date\n";
	    
	}
    }
}

    
# retrieve top lists parameters 

#DEBUG install a fake topList switch
$FORM{topVisitors} = 'on';

$topClientList = $FORM{topVisitors} eq 'on' ? 1 : 0; 

if ($topClientList == 1)
{
    $topClients = $FORM{ Numusers};
    
    if (($topClients ne "0" && int($topClients) == 0) 
	|| $topClients < 0)
    {
	$numErr++;
	$errMsg .= "$numErr - $formErr The number of Top Visitors must be positive or zero\n";
	
    }
    elsif ($topClients == 0)
    {
	# turn off the Top x Clients report
	$topClientList = 0;
    }
    else
    {
	# fetch the at least number 
	$atLeastClients = $FORM{Moreusers};
    
	if ((int($atLeastClients) == 0) 
	    || $atLeastClients < 0)
	{
	    $numErr++;
	    $errMsg .= "$numErr - $formErr The number of Least Access per Visitor must be one or
higher\n";
	
	}
    
    }
}

#DEBUG install a fake topList switch
$FORM{topFiles} = 'on';

$topFileList = $FORM{topFiles} eq 'on' ? 1 : 0; 

if ($topFileList == 1)
{
    $topFiles = $FORM{ Numfiles};
    
    if (($topFiles ne "0" && int($topFiles) == 0)
	|| $topFiles < 0)
    {
	$numErr++;
	$errMsg .= "$numErr - $formErr The number of Top Files must be positive or zero\n";
	    
    }
    elsif ( $topFiles == 0)
    {
	# turn off the Top x Files Report
	$topFileList = 0;
    }

    else
    {
	$atLeastFiles = $FORM{ Morefiles};
    
	if ((int($atLeastFiles) == 0) 
	    || $atLeastFiles < 0)
	{
	    $numErr++;
	    $errMsg .= "$numErr - $formErr The number of Least Access per Page must be one or higher\n";
				 
	}
    
    }
}


$topDomains = $FORM{ DOMVAL};
if (($topDomains ne "0" && int($topDomains) == 0)
    || $topDomains < 0)
{
    $numErr++;
    $errMsg .= "$numErr - $formErr The number of Top Companies must be positive or zero\n";
           
}
elsif( $topDomains > 0)
{
    $topDomainList = 1;
}
    
$topNets = $FORM{ NETVAL};
if (($topNets ne "0" && int($topNets) == 0)
    || $topNets < 0)
{
    $numErr++;
    $errMsg .= "$numErr - $formErr The number of Top Countries must be positive or zero\n";
    
}
elsif ($topNets > 0)
{
    $topNetList = 1;
}
    
$numReferrers = $FORM{NumReferrers};
if (($numReferrers ne "0" && int($numReferrers) == 0)
    || $numReferrers < 0)
{
    $numErr++;
    $errMsg .= "$numErr - $formErr The number of Top Referrer Pages must be positive or zero\n";
           
}
elsif( $numReferrers > 0)
{
    $referrerList = 1;
    $fXOwnReferrer = $FORM{XOwnReferrer} eq 'on' ? 1 : 0;
}
    
$browserList = $FORM{BrowserList} eq 'on' ? 1 : 0;
$platformList = $FORM{PlatformList} eq 'on' ? 1 : 0;
$byteHourlyAvg = $FORM {HourlyTransferbox} eq 'on' ? 1 : 0;
$byteDailyTotal = $FORM {DailyTransferbox} eq 'on' ? 1: 0;
$docTitles = $FORM {DocTitles} eq 'on' ? 1 : 0;
$hourlyAvg = $FORM{Hourbox} eq 'on' ? 1 : 0;
$dailyTotal = $FORM {Dailytotalbox} eq 'on' ? 1 : 0;
$dailyAvg = $FORM {Dailybox} eq 'on' ? 1 : 0;
$periodTotal = $FORM {Totalbox} eq 'on' ? 1 : 0;
$decompress = $FORM {Decompress} eq 'on' ? 1 : 0;

#v2.1.5 flag to keep cgi script with != arguments separate
$keepCGIArgs = $FORM {KeepCGIArgs} eq 'on' ? 1 : 0;

#v2.1.6 flag to classify cgi files as pages
$CGIPages = $FORM {CGIPages} eq 'on' ? 1 : 0;
 
#v2.1.5 request filter
$reqFilter = 0;
if ($FORM{KeepHTML} eq 'on')
{
	$reqFilter += 1;
}
if ($FORM{KeepCGI} eq 'on')
{
	$reqFilter += 2;
}
if ($FORM{KeepOther} eq 'on')
{
	$reqFilter += 4;
}

if ($reqFilter == 0)
{
    $numErr++;
    $errMsg .= "$numErr - $formErr You must select at least one of 'HTML', 'CGI' or 'Other'\n";
           
}
    
# exclusion filters: Clients or Files matching these strings
# will be left out from the report

$xClientList = $FORM {ClientFilter};

&FormatTextBox (*xClientList);
				
$numXClients = 0;
if ($xClientList ne '')
{
    $xClient = 1;
    @xList = split (/\n/, $xClientList);
    $numXClients = $#xList + 1;
    $xClientCase = $FORM{XClientCase} eq 'on' ? 1 : 0;
    $xClientMatch = $FORM{XClientMatch}; # radio button
    # NEW
    $xAllClientsBut = $FORM {XAllClientsBut}; # radio button
}

$xUnresolved = $FORM{XUnresolved} eq 'on' ? 1 : 0;
if ($xUnresolved == 1)
{
    #filter out unresolved addresses
    if ($numXClients > 0)
    {
	$xClientList .= "\n";
    }
    else
    {
	$xClient = 1;
	$xClientCase = 0;
	$xClientMatch = 0;
    }
    $xClientList .= "Unresolved Visitor Address";
    $numXClients ++;

    $fResolveIP = 0;
}
	
	

$xFileList = $FORM {FileFilter};

&FormatTextBox (*xFileList);
				
if ($xFileList ne '')
{
    $xFile = 1;
    @xList = split (/\n/, $xFileList);
    $numXFiles = $#xList + 1;
    $xFileCase = $FORM{XFileCase} eq 'on' ? 1 : 0;
    $xFileMatch = $FORM{XFileMatch}; # radio button
    # NEW
    $xAllFilesBut = $FORM {XAllFilesBut}; # radio button
}

$mClientList = $FORM{MatchClient};
				
&FormatTextBox (*mClientList);

if ($mClientList ne '')
{
    $matchClientList = 1;
    @xList = split (/\n/, $mClientList);
    $numMClients = $#xList + 1;
    $mClientCase = $FORM{MClientCase} eq 'on' ? 1 : 0;
    $mClientMatch = $FORM{MClientMatch}; #radio button

}

$mFileList = $FORM{MatchFile};

&FormatTextBox (*mFileList);

if ($mFileList ne '')
{
    $matchFileList = 1;
    @xList = split (/\n/, $mFileList);
    $numMFiles = $#xList + 1;
    $mFileCase = $FORM{MFileCase} eq 'on' ? 1 : 0;
    $mFileMatch = $FORM{MFileMatch}; # radio button
}
				
$mail = $FORM{DeliverMail} eq 'on' ? 1 : 0;
$emailTab = $FORM{EmailTab} eq 'on' ? 1: 0;
$web = $FORM{DeliverWeb} eq 'on' ? 1 : 0;
    
if ($web == 0 && $mail == 0 && $emailTab == 0)
{
    $numErr++;
    $errMsg .= "$numErr - $formErr You must select at least one delivery method.\n";
}

$numEmailAdd = 0;
				
#check email addresses
if ($mail == 1 || $emailTab == 1)
{
    $emailTo = $FORM{Address};

    &FormatTextBox (*emailTo);

    if ($emailTo ne '')
    {
	#check Email addresses, newline separated
	@emailAdds = split( /\n/, $emailTo);
	foreach $add (@emailAdds)
	{
		
	    #check address and exit with error msg
	    #if contains illegal chars
	    if ( $add !~ /^[a-zA-Z0-9_\-+\.\@%]+$/) 
	    {
		$numErr++;
		$errMsg .= "$numErr - $formErr e-mail address \"$add\" contains illegal character(s).\n";
		
	    }
	    else
	    { 
		$numEmailAdd++;
	    }
	}
    }
    else
    {
	$mail = 0;
	$emailTab = 0;
	$numErr++;
	$errMsg .= "$numErr - $formErr e-mail delivery selected with no address.\n";
    }
}

#check some configuration file parameters
if (!defined ($yourHost) || $yourHost !~ /\S+/)
{
    # pick a default display name for an unnamed site
    $yourHost = "<unnamed site>";
}

if (!defined ($dirSuffix) || $dirSuffix !~ /\S+/)
{
    # must have a prefix for default html page
    # these pages will not be accounted for otherwise
    $dirSuffix = "index.html";
}
 
   
############################  FILES SETUP ############################
@searchLog = split (/\n/, $matchLog);
$numStr = $#searchLog + 1;

$logPath = $accessLogPath;
if ($platform eq "OS_NT")
{
    $logPath =~ s/\\/\//g;
}

# look at each search pattern
# check for file matches in every parent directory
# accessLogPath is prepended when the search pattern does not start
# with a folder delimiter
%matchList = ();
%decompressedNames = ();
$addSecs = 0;
searchDir: 
foreach $pattern (@searchLog)
{
    if ($platform eq "OS_NT")
    {
	# change folder delimiter to unix format
	$pattern =~ s/\\/\//g;
    }

	# catch drive specifier on NT
	if ($platform eq "OS_NT" && $pattern =~ /^[a-zA-Z]:/)
	{
		$fullPath = $pattern;
	}
    elsif ($pattern !~ /^\//)
    {
		# partial path, does not start with a '/'
		# prepend the default log folder
		$fullPath = $logPath . $pattern;
    }
    else
    {
		# path start with a / (valid on NT, not drive specifier)
		$fullPath = $pattern;
    }
	
    # extract the parent folder
    $i = rindex ($fullPath, "/") + 1;
    $parentDir = substr ($fullPath, 0, $i);
    #extract file pattern 
    $filePattern =  substr ($fullPath, $i);
    
    # add wildcards to pattern if match subset is on
    if ($mLogMatch == 1)
    {
	$filePattern = "*" . $filePattern . "*";
    }

    # convert any metachar to '\metachar'
    $filePattern =~ s/(\W)/\\$1/g;

    # convert wildcard * to '.*'
    $filePattern =~ s/\\\*+/\.\*/g;

    # resolve all  ./ && ../
    $parentDir = &AliasDir ($parentDir);

    if (!opendir (DIR, $parentDir))
    {
		$numErr++;
		$errMsg .= sprintf ("$numErr - $sysErr Can't access directory '$parentDir':\n\t%s\n", $!);
	
		next searchDir;
    }

    #get a list of all files in directory
    @fileList = readdir (DIR);

    closedir (DIR);

  searchFile:
    foreach $file (@fileList)
    {
		next searchFile if ($file eq "." || $file eq "..");
		$fullName = $parentDir . $file;
	   
		if (&IsMatch ($filePattern, $file, $matchCase))
		{	
	    	if (&IsCompressed ($file) == 1)
	    	{
		
			next searchFile if ($decompress == 0 || 
					    $platform ne "OS_UNIX");
		
	                # get compressed file real name and size
			$command = "$ENV{COMPRESS_HOME}gzip -l $fullName";
			if (open (ZIP, "$command |"))
			{
			
				$gzGlob = ">";
				while (<ZIP>)
				{
					$gzGlob .= $_;
				}
				close (ZIP);
				@gzInfo = split (/\s+/, $gzGlob);
				$decompressedNames {$fullName} = substr ($gzInfo[7], rindex ($gzInfo[7], "/") + 1);
		
				
				$size = int ($gzInfo[5]);
			# add 1.5 seconds per megabyte to decompress
				$addSecs += ($size / 1048576) * 1.5;
	
			}
			else
			{
				$gzGlob[0] = "Can't pipe from gzip\n$!\n";
			}
	    	}
	    	else
	    	{
				next searchFile if (! &IsLogFile ($fullName));
				$size = -s $fullName;
	    	}
 
	    	# keep match list in a hash table
	    	# we only want one file name if several patterns
	    	# match the same file
	    	$matchList{$fullName} = $size;
	  
		}
    }
}

# make a newline delimited file list and collect sizes
$totalSize = 0;
$fileList = "";
$numFiles = 0;
foreach $file (sort keys (%matchList))
{
    $totalSize += $matchList{$file};

    # srengine expects a newline delimited list of 
    # complete path names to the files it will process
    $fileList .= $file . "\n";

    $numFiles ++;
}

if ($numFiles == 0 || $totalSize == 0)
{
	$numErr++;
	if ($numStr > 1)
	{
	    @mLog = split (/\n/, $matchLog);
	    $logStr = "";
	    foreach $log (@mLog)
	    {
			$logStr .= "\n\t\t$log"; 
	    }
	    
	}
	else
	{
	    $logStr = "'$matchLog' ";
	}
	$logStr .= "\n\t\t";
	$errMsg .= sprintf ("$numErr - $formErr No log file matching the pattern%s %swas found.", $numStr
> 1 ? "s" : "", $logStr);

}

if (!open (SETTINGS, ">$settingsFile"))
{
    $numErr++;
    $errMsg .= sprintf ("$numErr - $sysErr Can't open the SurfReport setting file:
'$settingsFile'\n\t%s\n",$!);
	
}
else
{
    chmod (0666, $settingsFile);
}
				
# exit with error message now !
if ($numErr > 0)
{
    &Error ($errMsg, $numErr);
    #erase useless settings files
    close (SETTINGS);
    unlink ($settingsFile);

    exit (-1);
}

#the time it should take
$secPerMB = 1.5;
$sec = ($totalSize / 1048576) * $secPerMB;
$sec += $addSecs;

# avoid the zero second reading 
if ($sec < 2)
{ 
    $sec = 2;
}
    
if ($sec >= 60)
{
    $min = int ($sec / 60);
    $dsec = int ($sec - ($min * 60));
    if ($min >= 60)
    {
		$hr = int ($min / 60);
		$min -= $hr * 60;
		if ($dsec > 30)
		{
	    	$min++;
		}
		$displayTime = sprintf("%d hour%s and %d minute%s", 
			       %hr, $hr > 1 ? "s" : "",
			       $min, $min > 1 ? "s" :"");
    }
    elsif ($dsec > 0)
    {
	
		$displayTime = sprintf ("%d minute%s and %d second%s", 
				$min, $min > 1 ? "s" : "",
				$dsec, $dsec > 1 ? "s" : "");
    }
    elsif ($dsec ==  0)
    {
	
		$displayTime = sprintf ("%d minute%s", 
				$min, $min > 1 ? "s" : "");
    }
}
else 
{
    $displayTime = sprintf ("%d second%s", $sec, $sec > 1 ? "s" :"");
}

if ($standAlone == 0)    
{
    $htmlBuf =  $httpHeader;
}
else
{
    $textBuf = "";
}

if ($web == 1)
{
    #insert a client pull meta tag for web reports 
    if ($noWait == 1)
    {
	$pull = $waitSR;
    }
    else
    {
	$pull = int($sec) + $waitSR;
    }
    
    
    # pull the monitoring appl.
    # the dispatcher gets the job previously held by srcheck (V2.0b7)
    $outputPath = $ENV{DISPATCHER_URL};
    
    $outputPath .= "/check/$nameBase/$tmpName";
    
  
    $htmlBuf .= "<META HTTP-EQUIV=\"Refresh\" CONTENT=\"";
    $htmlBuf .=  "$pull; URL=$outputPath\">";
	
}

$htmlBuf .= &ReportHeader("SurfReport Notification");
$textBuf .= "SurfReport Notification\n";
$textBuf .= $textLine;
$htmlBuf .= "<CENTER><P>\n";

$buf = sprintf ("SurfReport is now processing the following file%s:",
		$numFiles > 1 ? "s" : "");
$htmlBuf .= $buf;
$textBuf .= $buf;

#print out the list of files to be processed
#convert compressed file names into real file names
#sort the file list in alpha order
$htmlBuf .= "\n<PRE>\n";
$textBuf .= "\n";

@displayList = split (/\n/, $fileList);
$buf = "";
foreach $name (sort @displayList)
{
    $buf .= $name;
    if (defined ($cname = $decompressedNames{$name}))
    {
		# add real name of compressed file in paren
		$buf .= " ($cname)";
    }
    $buf .= "\n";
}
	
$htmlBuf .= $buf;
$textBuf .= $buf;

$htmlBuf .= "</PRE>\n";
$textBuf .= "\n";

# display size
$Kb = $totalSize / 1024;
if ($Kb >= 1024)
{
    $Mb = $Kb / 1024;
    $displaySize = sprintf ("%1.1f Mb", $Mb);
}
else
{
    $displaySize = sprintf ("%d Kb", $Kb);
}

$buf = sprintf ("Totalling %s of data.\n", $displaySize); 
$htmlBuf .= "<P>". $buf . "<P>";
$textBuf .= $buf;

if ($web == 1)
{  
    $buf = sprintf ("Your report will be automatically displayed in approximately %s.\n",
$displayTime);
    $htmlBuf .= $buf . "<P>";
    $textBuf .= $buf;

}

if ($mail == 1 || $emailTab == 1)
{
    $object = "";
    if ($mail == 1 )
    { 
	$object .= "report";
    } 
	
    if ($emailTab == 1)
    {
	if ($mail == 1)
	{
	    $object .= " and ";
	}
	$object .= "Tab Delimited file";
    }
	

    $buf = sprintf ("Your %s %s %sbeing e-mailed to the following address%s:\n", $object,
	   ($mail == 1 && $emailTab == 1) ? "are" : "is",		      
	    $web == 1 ? "also " : "", 
            $numEmailAdd > 1 ? "es" :"");
    $htmlBuf .= $buf . "<PRE>". $emailTo;
    $textBuf .= $buf. $emailTo;

    $htmlBuf .= "</PRE><P>\n";
    $textBuf .= "\n";

    if ($web == 0)
    {
	$buf = sprintf ("Expect delivery in about approximately %s.<P>", 
			     $displayTime);
	$htmlBuf .= $buf;
	$textBuf .= $buf;
    }
}

if ($fResolveIP == 1)
{
    # add a warning, don't know how long !
    $htmlBuf .= "<P>";
    $textBuf .= "\n";

    $buf = "You have chosen to resolve numerical addresses.";
    $htmlBuf .= $buf . "<BR>";
    $textBuf .= $buf . "\n";

    $buf = "This option may delay the delivery of your report.";
    $htmlBuf .= $buf . "<BR>";
    $textBuf .= $buf . "\n";

    $buf = "(setting 'fResolveIP' to 0 in the configuration ";
    $buf .= "file will turn off this feature)";
    $htmlBuf .= $buf;
    $textBuf .= $buf . "\n";
}

$htmlBuf .= "<P>";
$abortURL = $ENV{DISPATCHER_URL};

$abortURL .= "/check/$nameBase/$tmpName/-2";
$htmlBuf .= "<H4>&gt; <A HREF=\"$abortURL\">Abort</A> &lt;</H4>";
$htmlBuf .= "<p>";

$buf = "Thank you for using SurfReport!";
$htmlBuf .= "<P>" . $buf . "<P>";
$textBuf .= "\n" . $buf . "\n";

$htmlBuf .= $htmlCopyright;
$textBuf .= $textCopyright;


#DEBUG print env
#$printEnv = 1;
#$printForm = 1;
if ($printEnv == 1)
{
    $htmlBuf .= "</CENTER><P><PRE>";
    foreach $key (sort keys (%ENV))
    {
	$htmlBuf .= "$key:$ENV{$key}\n";
	$textBuf .= "$key:$ENV{$key}\n";
    }
    $htmlBuf .= `pwd`;
    $htmlBuf .= "</PRE><CENTER>";
    $textBuf .= "\n";
}
#DEBUG print From data
if ($printForm == 1)
{
    $htmlBuf .= "</CENTER><P><PRE>";
    while (($key,$val) = each (%FORM))
    {
	$htmlBuf .= "$key:$val\n";
	$textBuf .= "$key:$val\n";
    }
    $htmlBuf .= "\nfile search pattern:$pattern\n";
    $htmlBuf .= "\nfile search array:@searchLog\n";
    $htmlBuf .= "\nnumber of search patterns:$numStr\n";
    $htmlBuf .= "\nfile match array:@mLog\n";
    $htmlBuf .= "\nRaw Form Data:\n$formData\n";
   
    $htmlBuf .= "</PRE><CENTER>";
}



# save settings for the C code

#obsolete V2.07B
$srFolder = $surfOut;
$gifFolder = $surfOut . "gifs/";
#V2.0B7
print SETTINGS "nameBase=$nameBase&";
print SETTINGS "numLogs=$numFiles&";
print SETTINGS "logFiles=$fileList&";
print SETTINGS "matchLog=$matchLog&";
print SETTINGS "deliverWeb=$web&";
print SETTINGS "deliverMail=$mail&";
print SETTINGS "decompress=$decompress&";
				
if ($mail == 1 || $emailTab == 1)
{
    print SETTINGS "numAdds=$numEmailAdd&";
    if ($numEmailAdd > 0)
    {
	print SETTINGS "emailTo=$emailTo&";
    }
    if ($emailTab == 1)
    {
	print SETTINGS "emailTab=$emailTab&";
    }
}
if ($topClientList == 1)
{
    print SETTINGS "topClientList=top&";
    print SETTINGS "numClients=$topClients&";
    print SETTINGS "leastClients=$atLeastClients&";
   
}
if ($topFileList == 1)
{
   print SETTINGS "topFileList=top&";
   print SETTINGS "numFiles=$topFiles&";
   print SETTINGS "leastFiles=$atLeastFiles&";
    
}
if (defined ($topDomainList))
{
    print SETTINGS "topDomains=$topDomains&";
}
if (defined ($topNetList))
{
    print SETTINGS "topNets=$topNets&";
}

if (defined ($referrerList))
{
    print SETTINGS "numReferrers=$numReferrers&";
    print SETTINGS "xOwnReferrer=$fXOwnReferrer&";

    $xRefList = "";
    foreach $name (@XReferrers)
    {
	$xRefList .= $name."\n";
    }
    print SETTINGS "xReferrers=$xRefList&";

}
if ($browserList == 1)
{
    print SETTINGS "browserList=$browserList&";
}
if ($platformList == 1)
{
    print SETTINGS "platformList=$platformList&";
}


if ($realDomainName ne "")
{
    print SETTINGS "realDomainName=$realDomainName&";
}
				
if (defined ($xClient))
{
    print SETTINGS "numXClients=$numXClients&";
    print SETTINGS "xClients=$xClientList&";
    print SETTINGS "xClientCase=$xClientCase&";
    print SETTINGS "xClientMatch=$xClientMatch&";
    print SETTINGS "xAllClientsBut=$xAllClientsBut&";
}

print SETTINGS "xUnresolved=$xUnresolved&";

if (defined ($xFile))
{
    print SETTINGS "numXFiles=$numXFiles&";
    print SETTINGS "xFiles=$xFileList&";
    print SETTINGS "xFileCase=$xFileCase&";
    print SETTINGS "xFileMatch=$xFileMatch&";
    print SETTINGS "xAllFilesBut=$xAllFilesBut&";
}
if (defined ($matchClientList))
{
    print SETTINGS "numMClients=$numMClients&";
    print SETTINGS "matchClients=$mClientList&";
    print SETTINGS "mClientCase=$mClientCase&";
    print SETTINGS "mClientMatch=$mClientMatch&";
}
if (defined ($matchFileList))
{
    print SETTINGS "numMFiles=$numMFiles&";
    print SETTINGS "matchFiles=$mFileList&";
    print SETTINGS "mFileCase=$mFileCase&";
    print SETTINGS "mFileMatch=$mFileMatch&";
}
 
print SETTINGS "startDate=$startTime&";
print SETTINGS "endDate=$endTime&";
    
print SETTINGS "hourlyAvg=$hourlyAvg&";
print SETTINGS "dailyAvg=$dailyAvg&";
print SETTINGS "dailyTotal=$dailyTotal&";
print SETTINGS "periodTotal=$periodTotal&";
print SETTINGS "docTitles=$docTitles&";
print SETTINGS "keepCGIArgs=$keepCGIArgs&";
print SETTINGS "CGIPages=$CGIPages&";
print SETTINGS "reqFilter=$reqFilter&";

print SETTINGS "byteHourlyAvg=$byteHourlyAvg&";
print SETTINGS "byteDailyTotal=$byteDailyTotal&";

print SETTINGS "dirPrefix=$dirPrefix&";
print SETTINGS "dirSuffix=$dirSuffix&";
print SETTINGS "hostName=$yourHost&";

if ($visitMaxIdle < 0)
{
    # reset to one hour if user setting is illegal
    $visitMaxIdle = 60;
}
print SETTINGS "visitMaxIdle=$visitMaxIdle&";
				
$aliasList = "";
$i = 0;
while (($key, $val) = each %Aliases)
{
    $aliasList .= $key.",".$val."\n";
    $i++;
}
print SETTINGS "numAlias=$i&";
print SETTINGS "aliases=$aliasList&";
 

$tocList = "";
$i = 0;
foreach $name (@TocNames)
{
    $tocList .= $name."\n";
    $i++;
}
print SETTINGS "numNames=$i&";
print SETTINGS "tocNames=$tocList&";


$cgiExtList = "";
foreach $name (@CGIExtensions)
{
    $cgiExtList .= $name."\n";
}
print SETTINGS "cgiExt=$cgiExtList&";

print SETTINGS "xHeadReq=$xHeadReq&";

print SETTINGS "converterApp=$converterApp&";

close (SETTINGS);

#build a full path to the C engine	
$programName = $ENV{SURFREPORT_HOME};
$programName .= "cgi/";
$programName .= $coreApp;
			
if ($standAlone == 0 && $platform eq 'OS_UNIX')
{
    # make the c program run in the bg
    #   by doing a fork 
   	
    $pid = fork();
    if ($pid == -1)
    {
	&Error ("Could not fork:\n\t".$!, 1);
	exit;
    }
    if ($pid)
    {				

	# our parent process
	# just needs to print its output
        $htmlBuf .= "\n</CENTER></BODY></HTML>";
	print $htmlBuf;  
	
	exit (0);
    }
    else
    {
  
	# run online
	
	@commandLine = ($programName, $settingsFile);
	
	# flush stdout
	# needed to display the notice page right away
	# and close the http connection
	close (STDOUT);
	close (STDIN);
	close (STDERR);
	
	#wait for parent to die 
	sleep 1 until getppid == 1;
	
	#execute SR engine in the background
	exec @commandLine;
   
    }
}
elsif ($standAlone == 1 || $platform ne 'OS_UNIX')
{
    if ($standAlone == 1)
    {
    	if (!defined ($ENV{SURFREPORT_DIRECTOUTPUT}))
        {
            print $textBuf;
        }
    }
    else
    {
		print $htmlBuf;
    }

    @commandLine = ($programName, $settingsFile);

	if ($platform eq "OS_NT")
	{
		# flush stdout
		# needed to display the notice page right away
		# and close the http connection
		close (STDOUT);
		close (STDIN);
		close (STDERR);

		exec @commandLine;
	}
	else
	{
		system @commandLine;
	}

}

######################### SUBROUTINES ############################

sub GetFormInput
{
    local (@pairs, $buffer, $name, $value);
 
    if ($ENV{'REQUEST_METHOD'} eq 'POST')
    {
	read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
	@pairs = split( /&/, $buffer);
	
	foreach $pair (@pairs)
	{
	    ($name, $value) = split(/=/, $pair);
	    $value =~ tr/+/ /;
	    $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
	    $value =~ s/\r//g;
	    $FORM{$name} = $value;
	}
    }
    return $buffer;
       
}

sub GetFormParamFile
{
    local ($fileName) = @_;
    local ($name, $value);

    open (IN, $fileName) || 
	die "Can't open form parameter file '$fileName':\n\t$!\n";

    while(<IN>)
    {
    	# skip comment lines (optional spaces and #)
    	next if (/^\s*#/);
		chop;
		
		($name, $value) = split (/,/, $_, 2);
		$FORM{$name} = $value;
    }
    close (IN);


}

#accept commas, spaces, tabs & newlines as separators
#format the input so every item
#is separated by a single newline
sub FormatTextBox
{
    local (*text) = @_;
    
    $text =~ s/[,\s]+/\n/g;
    $text =~ s/^\n*//;
    $text =~ s/\n*$//;
}

sub TimeDateToStamp
{
    local($year, $month, $day) = @_;
    local ( $timestamp);

    $month = &MonthToNum( $month);

    if ($day < 0 || $day > 31 || $year < 1900 || $year > 3000)
    {
	$fCorruptedEntry = 1;
	return 0;
    }

    $timestamp =  $day + ( $month * 32) + (( $year - 1990) * 500);
}

sub MonthToNum
{
    local ($m) = @_;

    if ( $m eq 'Jan' || $m eq '1')
    {
	return 1;
    }
    elsif ( $m eq 'Feb'|| $m eq '2')
    {
	return 2;
    }
    elsif ( $m eq 'Mar'|| $m eq '3')
    {
	return 3;
    }
    elsif ( $m eq 'Apr'|| $m eq '4')
    {
	return 4;
    }
    elsif ( $m eq 'May'|| $m eq '5')
    {
	return 5;
    }
    elsif ( $m eq 'Jun'|| $m eq '6')
    {
	return 6;
    }
    elsif ( $m eq 'Jul'|| $m eq '7')
    {
	return 7;
    }
    elsif ( $m eq 'Aug'|| $m eq '8')
    {
	return 8;
    }
    elsif ( $m eq 'Sep'|| $m eq '9')
    {
	return 9;
    }
    elsif ( $m eq 'Oct'|| $m eq '10')
    {
	return 10;		
    }
    elsif ( $m eq 'Nov'|| $m eq '11')
    {
	return 11;
    }
    elsif ( $m eq 'Dec'|| $m eq '12')
    {
	return 12;
    }
    else
    {			       
	$fCorruptedEntry = 1;
	return 0; 
    }				
   
} 

sub IsValidDate
{
    local ($y, $m, $d) = @_;
    local ($maxDays);

    $m = &MonthToNum( $m);
    
    if (($m == 2) && (&IsLeapYear( $y)))
    {
        $maxDays = 29;
    }
    else
    {
	$maxDays = $DaysInMonth[ $m - 1];
    }

    $d <= $maxDays;
}
	
sub IsLeapYear
{
    local ( $y) = @_;
   
    $y % 4 == 0;

}


sub PostTextError
{
    local ($errorMsg, $numErr) = @_;
   
    printf ("Sorry, SurfReport Found $numErr Error%s:\n", $numErr > 1 ? "s": "");

    print "\n$errorMsg\n";
    
    print "Please correct and try again!\n";
    print "\n";
    
}

sub Error
{
    
    local ($errorMsg, $numErr) = @_;
    if ($standAlone == 1 && !defined ($ENV{SURFREPORT_DIRECTOUTPUT}))
    {
	&PostTextError($errorMsg, $numErr);
    }
    else
    {				
	&PostError($errorMsg, $numErr);
    }
}
    

sub IsMatch
{
    local ($pattern, $file, $matchCase) = @_;

    if ($matchCase == 1)
    {
	return ($file =~ /^$pattern$/);
    }
    else
    {
	return ($file =~/^$pattern$/i);
    }
    
}

sub AliasDir
{
    local ($dir) = @_;
    local ($hasRef);

    #remove reference to own directory
    $dir =~ s/\/\.\/+/\//g;
    $hasRef = 1;
    while ($hasRef == 1)
    {
    	# look for backward ref and resolve them
		if ($dir !~ s/\/\w+[^\/]*\/\.\.(\/.*\/)/$1/)
		{			
		    $hasRef = 0;
		}
	
    }  
    return $dir;
}

sub IsCompressed
{
    local ($file) = @_;
    local ($ext, $i);

    # compare extension
    $i = rindex ($file, ".");
    if ($i == $[ - 1)
    {				
	# no extensions
	return 0;
    }
    $ext = substr ($file, $i+1);
    
    if ($ext eq "gz" || $ext eq "Z")
    {
	# only support UNIX gzip/compress files
	# NT zip soon
	return 1;
    }
    return 0;
    
}

sub IsLogFile
{
    local ($file) = @_;
    local ($n, $isLog);

    if (!open (LOG, $file))
    {
	return 0;
    }

    $n = 0;
    $isLog = 0;

  IsLog:
    while (<LOG>)
    {
        #something that, at least, looks like a log entry
	if (/^\S*.*\[\S*.*\].*\".*\"\s*\d*.*$/)
	{
		# CLF format
	    $isLog = 1;
	    last IsLog;
	}
	elsif ((@params = split (',', $_)) && $#params >= 10)
	{
		# MS format 
		$isLog = 1;
		last IsLog;
	}

	# try it ten times, some logs have headers 
	# or blank lines at the top
	last IsLog if (++$n >= 10);
    }
    close (LOG);
    return $isLog;
}











