#!/usr/bin/perl


########################### SRCHECK ###########################
#
# check if the html output file is ready for display
#
# V 2.1.7 Single User
#
# Author Michel G Delory
# Created : May 1996
#
# Last Revision: Nov 30 1997
#
# copyright (c) BienLogic, Inc 1996, 1997
#
#

$httpHeader = "Content-type: text/html\n\n";
$errorHeader = "<HTML><HEAD><TITLE>SurfReport Error</TITLE></HEAD>";
$converterApp = "srreport.cgi";
$copyright =    "<H5>SurfReport 2.1.7<br>Copyright &copy; 1996,1997 <A HREF = \"http://www.bienlogic.com\">BienLogic, Inc.</A></H5>";

####################### ENVIRONMENT ########################

if (!defined ($ENV{SURFREPORT_CONFIG}) ||
    !defined ($ENV{SURFREPORT_LIB}))
{
    #can't acces config file, forget it
    #can't access error routines in config hence this
    $err = "SurfReport environment is not available.\n";
    $err .= "Can't access your configuration file\n";
    $err .= "You must run this script through the SurfReport dispatcher\n";

    print $httpHeader;
    print $errorHeader;
    print "<body BGCOLOR = #FFFFFF TEXT = #OOOOOO >\n";
    print "<H3>Environment Error</H3>\n";
    
    print "<pre>\n\n";
    print $err;
    print "\n\n</pre>";

    print $copyright;

    print "</BODY></HTML>";
    exit (-1);

}

require $ENV{SURFREPORT_CONFIG};
require $ENV{SURFREPORT_LIB};

#get the base path for all SR output files
#and make sure all needed env vars are available
if (!defined ($ENV{SURFREPORT_OUT}) ||
    !defined ($ENV{SURFREPORT_HOME}) ||
    !defined ($ENV{DISPATCHER_URL}) ||
    !defined ($platform = $ENV{PLATFORM}))
{
    # exit now if env vars not available
    $err = "SurfReport environment is not available.\n";
    $err .= "You must run this script through the SurfReport dispatcher\n";

    &Error($err);
    
    exit (-1);

}

if ($platform eq "OS_NT")
{
	$coreApp = "srengine.exe";
	$chartApp = "srchart.exe";
}
else
{
	$coreApp = "srengine.cgi";	
	$chartApp = "srchart.cgi";
}

####################### INIT ###########################

# $lookAgainDelay in surf.cf
# default is 10 secs
# make sure this parameter falls between 
# reasonable values
$waitMore = int($lookAgainDelay);
if ($waitMore < 5)
{
    $waitMore = 5;
}
elsif ($waitMore > 60)
{
    $waitMore = 60;
}

# V2.0B7 get the temporary settings file name from the command line
$nameBase = $ARGV[0];
$tmpName = $ARGV[1];
$count = $ARGV[2];
#($tmpName, $count) = split (/&/, $ARGV[0]);

$dispatcher = $ENV{DISPATCHER_URL};

if (!defined ($count))
{
    $count = 0;
}

#path to SR app folder
$appDir = $ENV{SURFREPORT_HOME};
$appDir .= "cgi/";


# the name of the temporary settings file generated by the perl front end
$tempFile = $ENV{SURFREPORT_OUT} . "tmp/";
$tempFile .= $tmpName;

# other files generated by the C engine
$errorFile = $tempFile.".err";
$runFile = $tempFile.".txt";
$logFile = $tempFile.".log";
$validationFile = $tempFile . ".val";
$pidFile = $tempFile . ".pid";

# V2.09
# go thru dispatcher to serve report when user does not
# validate the copy
# this gives us a chance to get rid of the val file
if ($count == -1)
{
    unlink ($validationFile);
}

if (-e $tempFile)
{
    if (open (TMP, "$tempFile"))
    {
        $fIsTemp = 1;
        close (TMP);
    }
    else
    {
	$fIsTemp = 0;
    }
}

if ($count == -2)
{
    # attempt to abort processing

    # write http header, always needed
    $htmlBuf = $httpHeader;

    $htmlBuf .= &ReportHeader ("SurfReport Termination");
	
    $htmlBuf .= "<H3>SurfReport Termination status</H3>";
    $htmlBuf .= "<PRE>";

    $n = 0;
    while (!-e $pidFile && $n < 5)
    {
	# give process a chance to write the pid file 
	sleep (1);
	$n++;
    }

    if (!-e $pidFile)
    {
	$htmlBuf .= "Error:\nCan't find file '$pidFile'.\n";
	if (!-e $tempFile)
	{
	    $htmlBuf .= "Your report may already be done.\n";
	}
	else
	{
	    $htmlBuf .= "The processing engine may not have started.\n";
	}
	
    }
    elsif (!open (PID, $pidFile))
    {
	$htmlBuf .= "Error:\nCan't open file '$pidFile'.\n$!\n";
	
    }
    else
    {
	$pid = <PID>;
	close (PID);

	if ($ENV{'PLATFORM'} eq "OS_UNIX")
	{
	   $cnt = kill 'TERM', $pid;
	   if ($cnt == 0)
	   {
	       $htmlBuf .= "Error:\nCan't send the 'TERM' signal \n";
	       $htmlBuf .= "to $coreApp\n.";
	   }
	   else
	   {
	       $htmlBuf .= "$coreApp terminated.\n";
	   }
	}
	elsif ($ENV{'PLATFORM'} eq "OS_NT")
	{
	   $command = "kill.exe $pid > $pidFile"; 
	   system $command;
	   $htmlBuf .= "$coreApp terminated.\n";
	   unlink ($pidFile);
	   
	}
	else
	{
	    $htmlBuf .= "Error:\nSorry, ";
	    $htmlBuf .= "this function is not yet supported on your platform.\n";
	}
    }

    # clean up tmp files
    unlink <$tempFile*>;
    unlink ($tempFile);

    $htmlBuf .= "</PRE>";
    $htmlBuf .= "<P>";
    $htmlBuf .= &InterfaceLink();

    $htmlBuf .= "<P>Thank you for using SurfReport!<P>";
    $htmlBuf .= $copyright;
    $htmlBuf .= "\n</CENTER></BODY></HTML>";

}
elsif ( $fIsTemp == 1)
{
    # write http header, always needed
    $htmlBuf = $httpHeader;

    # temp file still exists
    # look for an error file the C code might have generated
    # it has the same name with the extension .err added
    
    
    # will look for a file the C program creates first thing
    # if this file does not exist, the C program probably
    # did not even run    
    

    if ( -e $errorFile)
    {
	# there has been a fatal error caught in the C code
	
	$htmlBuf .= &ReportHeader ("SurfReport Fatal Error");
	
	$htmlBuf .= "<H3>SurfReport has encountered a fatal error:</H3>";
	$htmlBuf .= "<PRE>";

	# debug , wait around until core app is finished with it!
	sleep (2);

	# open the error file
	if (!open (ERROR_FILE, $errorFile))
	{
	    # can't even open the darn thing
	    $htmlBuf .= "Sorry, <B>SurfReport</B> ";
	    $htmlBuf .= "can't retrieve the error message:\n";
	    # print system error (why can't open the file)
	    $htmlBuf .= "\t$!\n";
	    
	}
	else
	{
	    #lets print the error message, hoping it is self explanatory
	    while (<ERROR_FILE>)
	    {
		$htmlBuf .= $_;
	    }			
	    close (ERROR_FILE);
	
        }

	
	# clean up the srout/tmp/ folder
	# from all tmp files belonging to this session
	# the C code cleans this up if there is no error
	unlink <$tempFile*>;
	unlink ($tempFile);

	$htmlBuf .= "</PRE>";
	$htmlBuf .= "<A HREF=\"mailto:surf\@bienlogic.com\">Customer Support</A>";


	$htmlBuf .= "<P>";
	$htmlBuf .= &InterfaceLink();


    } # endif (-e errorFile)

    elsif ( !-e $runFile &&  $count > 4)
    {

	$htmlBuf .= &ReportHeader ("SurfReport Error");
	
	$htmlBuf .= "<H3>SurfReport has encountered a configuration error:";
	$htmlBuf .= "</H3>";
	$htmlBuf .= "<PRE>";

	$htmlBuf .= "<b>$coreApp</B>, the <B>SurfReport</B> core program ";
	$htmlBuf .= "didn't seem to run at all.\n";

	$corePath = $appDir.$coreApp;

	if (-e $corePath)
	{
	    $perm = 0;
	    if (-r $corePath)
	    {
		$perm = 4;
	    }
	    if (-x $corePath)
	    {
		$perm += 1;
	    }
	    if ($perm != 5)
	    {
		
		if (chmod (0755,$corePath) == 1)
		{ 
		    $htmlBuf .= "The permissions for <b>$coreApp</b> were set";
		    $htmlBuf .= " incorrectly.\n";
		    $htmlBuf .= "Permissions are now set to 0755...\n";
		}
		else
		{
		    $htmlBuf .= "The permissions are set incorrectly: ";
		    $htmlBuf .= "<b>$coreApp</b> needs Web read and execute";
		    $htmlBuf .= " permissions.\n";
		    $htmlBuf .= "<b>SurfReport</B> was unable to change them ";
		    $htmlBuf .= "for you so please, ";
		    $htmlBuf .= "cd to $appDir and type:\n";
		    $htmlBuf .= "<b>chmod 0755 $coreApp</B> at the prompt ";
		    $htmlBuf .= "and try again!\n";
		}
	    } # endif (perm != 5)
	    else # correct perm, other cause...
	    {
		
		$htmlBuf .= "Make sure you have the correct binaries";
		$htmlBuf .= "for your machine\n";
			
	    } # endif correct perms, wrong path
	} # endif (-e corePath)
	else
	{
	    $htmlBuf .= "<B>$coreApp</B> is not present in your ";
	    $htmlBuf .= "cgi-bin directory.\n";
	    $htmlBuf .= "Please make a copy of it from ";
	    $htmlBuf .= "your distribution cgi directory to the  $appDir ";
	    $htmlBuf .= "directory\nand try again!";
	}

	$htmlBuf .= "<P>";
	$htmlBuf .= &InterfaceLink();

	# clean up the srout/tmp/ folder
	# from all tmp files belonging to this session
	# the C code cleans this up if there is no error
	unlink <$tempFile*>;
	unlink ($tempFile);

    } # endif (!-e runFile && $count > 4)
    elsif ((-e $runFile && !-e $logFile) && $count > 4)
    {
	$htmlBuf .= &ReportHeader ("SurfReport Error");
	
	$htmlBuf .= "<H3>SurfReport has encountered a configuration error:";
	$htmlBuf .= "</H3>";
	$htmlBuf .= "<PRE>";

	$htmlBuf .= "<b>$converterApp</B>, the utility that writes HTML and ";
	$htmlBuf .= "text reports\ndidn't seem to run.\n";

	$converterPath = $appDir.$converterApp;

	if (!-e $converterPath)
	{
	    $htmlBuf .= "<B>$converterApp</B> is not present in your ";
	    $htmlBuf .= "cgi-bin directory.\n";
	    $htmlBuf .= "Please make a copy of it from ";
	    $htmlBuf .= "your distribution cgi directory to the $appDir ";
	    $htmlBuf .= "directory\nand try again!";
	}
	elsif (!-r $converterPath || !-x $converterPath)
	{
	   $htmlBuf .= "The permissions are set incorrectly: ";
	   $htmlBuf .= "<b>$converterApp</b> needs Web read and execute";
	   $htmlBuf .= " permissions.\n";
	   $htmlBuf .= "Please, cd to $appDir and type:\n";
	   $htmlBuf .= "<b>chmod 0755 $converterApp</B> at the prompt ";
	   $htmlBuf .= "and try again!\n";
	}
	else
	{
	    $htmlBuf .= "<b>$converterApp</b> was invoked using this line:\n";
	    $htmlBuf .= "<b>". $ENV{PERL_HOME};
	    $htmlBuf .= " " .$converterPath;
	    $htmlBuf .= "</B>\n";
	}

	$htmlBuf .= "<P>";
	$htmlBuf .= &InterfaceLink();

	# clean up the srout/tmp/ folder
	# from all tmp files belonging to this session
	# the converter app cleans this up if there is no error
	unlink <$tempFile*>;
	unlink ($tempFile);

    }
    else
    {
	# there is no error file or error file is busy
	# wait a little longer, send a refresh command

	if (!-e $runFile || (-e $runFile && !-e $logFile))
        {	
	   $count ++;
        }

	$outputPath = $dispatcher;
	$outputPath .= "/check/$nameBase/$tmpName/$count";

	$htmlBuf .= "<META HTTP-EQUIV=\"Refresh\" CONTENT=\"";
	$htmlBuf .=  "$waitMore; URL=$outputPath\">";


	$htmlBuf .= &ReportHeader("SurfReport Delay");
	$htmlBuf .= "<CENTER>";
	$htmlBuf .= "<B>SurfReport</B> ";
	$htmlBuf .= "is taking a little longer than expected.<P>";
	$htmlBuf .= "Your server may be busy at this time...<P>";
	$htmlBuf .= "<B>SurfReport</B> will try to fetch your report again in a few seconds, please wait.<P>";

	$abortURL = $dispatcher;
	$abortURL .= "/check/$nameBase/$tmpName/-2";
	$htmlBuf .= "<H4>&gt; <A HREF=\"$abortURL\">Abort</A> &lt;</H4>";

    } # endif no error file, existing run file, count < 4

    # print end of page
    
    $htmlBuf .= "<P>Thank you for using SurfReport!<P>";
    $htmlBuf .= $copyright;
    $htmlBuf .= "\n</CENTER></BODY></HTML>";

}
elsif (-e $validationFile && $count != -1)
{
    # report done, but this copy is not validated
    # offer a form for a user to enter the validation code
    # the form is generated here rather than in the C validation
    # program, just to keep it small
    # the validation program will serve the report whether or not
    # the code is valid (SR stops running after 30 days of this...)
    # if count is -1 we've just been thru this, the val file should have been
    # erased, but some servers will be slow at it

    $htmlBuf = $httpHeader;
    $htmlBuf .= &ReportHeader ("SurfReport Validation");

    $htmlBuf .= "<h3>Surfreport Validation</h3>";
    $htmlBuf .= "<PRE>\n";

    $action = $dispatcher;
    $action .= "/validate/$tmpName";
    $htmlBuf .= "<FORM ACTION=\"$action\" METHOD=POST>";

    # hidden value, the validation code will locate the report and serve it
    $location = $srOutURL.$nameBase.".html";
    $htmlBuf .= "<INPUT TYPE=HIDDEN NAME=\"Location\" VALUE=$location>";

    $htmlBuf .= "Please, enter your validation code ";
    $htmlBuf .= "<INPUT TYPE=TEXT SIZE=15 NAME=\"Key\">";
    $htmlBuf .= "<INPUT TYPE=submit VALUE=\"validate\">";
    $htmlBuf .= "</FORM>\n\n";
    $htmlBuf .= "If you don't have your validation code yet, just click ";
    # v2.09 go thru dispatcher to serve report
    # better with cache problems and erase val file
    $dispatcherCommand = $dispatcher ."/check/$nameBase/$tmpName/-1";
    $htmlBuf .= "<a HREF =\"$dispatcherCommand\">here</a> ";
    $htmlBuf .= "to get your report.\n";
    $htmlBuf .= "SurfReport will run unvalidated for 30 days...\n\n";
    $htmlBuf .= &InterfaceLink();
    $htmlBuf .= "&nbsp;<P>Thank you for using SurfReport!<P>";
    $htmlBuf .= "&nbsp;" . $copyright;
    $htmlBuf .= "\n</CENTER></BODY></HTML>";
    
}
else
{
    # temporary settings are gone, fetch the report

    # let the report generating script die
    sleep (1);
	
    $htmlBuf = "Location: ";
    $htmlBuf .= $srOutURL.$nameBase.".html";
    $htmlBuf .= "\n\n";
}


print $htmlBuf;


sub Error
{
    
    local ($errorMsg, $sysError) = @_;
    $errorMsg .= "\n";
    if (defined ( $sysError))
    {
	$errorMsg .= "The system reported this error: ";
	$errorMsg .= $sysError;		
    }
    &PostError($errorMsg, 1);
    
}




