#!/usr/bin/perl

################################ SRDESIGN #####################################
#
# V 2.1.7 Single User
#
# Generate an html FORM for customising charts created by srchart.cgi 
# 
#
# The name of the chart settings file is passed as argument (QUERY_STRING)
# an optional archived file name can also be passed in the query string
# 
#
# Author : Michel G. Delory
# created: April 1996
# last rev: Nov 29 1997
# 
# 
# copyright BienLogic Inc.
#
########################### ARRAYS & STRINGS ################################

$httpHeader = "Content-type: text/html\n\n";
$errorHeader = "<HTML><HEAD><TITLE>SurfReport Error</TITLE></HEAD>";
$htmlLine = "<P>\n<HR WIDTH = 80%>\n";  

$htmlTitleH = "<P><FONT SIZE = +2>";
$htmlTitleT = "</FONT><BR><PRE>\n";
$htmlEndReport = "</PRE>".$htmlLine;


@ChartTypes = ('bar',
	       'line',
	       'area', 
	       'scatter', 
	       'pie');


@ChartOrientations = ('horizontal',
		     'vertical');

@BoxPositions = ('topLeft',
		 'topCenter',
		 'topRight',
		 'sideLeft',
		 'sideRight',
		 'botLeft',
		 'botCenter',
		 'botRight');

@Positions = ('outside',
	      'inside',
	      'middle',
	      'none');

%Colors = ('Yellow', 5,
	   'Blue', 248,
	   'Red', 71,
	   'Lime Green', 119,
	   'Dark Blue', 213,
	   'Bright Blue', 186,
	   'Aqua', 110,
	   'Brick Red', 107,
	   'Magenta', 32,
	   'Crimson', 34,
	   'Brown', 250,
	   'Ochre', 137,
	   'Mustard', 53,
	   'Pumpkin', 23,
	   'Bright Orange', 29,
	   'Pink', 20,
	   'Chartreuse', 75,
	   'Bright Green', 77,
	   'Violet', 48,
	   'Lavender', 96,
	   'Purple', 249,
	   'Lavender', 96);

############################## ENVIRONMENT ##################################

if ($ENV{STAND_ALONE} eq "YES")
{
    $standAlone = 1;
}
else
{
    $standAlone = 0;
}


if (!defined ($ENV{SURFREPORT_CONFIG})||
    !defined ($ENV{SURFREPORT_LIB}))
{
    #V2.0B7
    #can't acces config file, let's go

    $err = "SurfReport environment is not available.\n";
    $err .= "Can't access your configuration file\n";
    $err .= "You must run this script through the SurfReport dispatcher\n";

    print $httpHeader;
    print $errorHeader;
    &FatalError ($err);

    exit (-1);

}

require $ENV{SURFREPORT_CONFIG};
require $ENV{SURFREPORT_LIB};

#get the base path for all SR output files
#and make sure all needed env vars are available
if (!defined ($surfOut = $ENV{SURFREPORT_OUT}) ||
    !defined ($ENV{SURFREPORT_HOME}) ||
    !defined ($ENV{DISPATCHER_URL}) )
{
    # exit now if env vars not available
    $err = "SurfReport environment is not available.\n";
    $err .= "You must run this script through the SurfReport dispatcher\n";

    print $httpHeader;
    print $errorHeader;
    &FatalError ($err);

    exit (-1);

}

#get command line arguments
$nameBase = $ARGV[0];
$setFile = $ARGV[1];

################################# FILES ##################################

#build complete path name for the files we'll need
$settingsFile = $surfOut . "set/" . $setFile;
$tocFileName = $surfOut . $nameBase . ".toc";
$dataFileName = $surfOut . $nameBase . ".txt";


# opens the settings file an put all its values 
# in the associative array %Settings
&GetSettings ($settingsFile);

# fetch the series names

unless (open (TOC, "$tocFileName"))
{
    &Error ("Can't open table of content file '$tocFileName'", $!);
    exit;
}

# retrieve info we need about series names
$fFound = 0;
 searchToc:
    while(<TOC>)
    {
	#tab delimited file of format
	#<report title><tab><position in the data file><newline>

	($title, $pos) = split (/\t/);
	if ($title eq $Settings{reportTitle})
	{
	    $fFound = 1;
	    last searchToc;
	}

    }
close (TOC);

if ($fFound == 0)
{
    &Error ("Can't find Data for Report untitled :'$Settings{reportTitle}'");
    exit;
}

# opens the tab delimited data file
unless (open (DATA, "$dataFileName"))
{
    &Error ("Can't open data file '$dataFileName'", $!);
    exit;
}

#position the data file 
seek (DATA, $pos, 0);
$dummy = <DATA>;
if ($Settings{hasSeriesLabels} eq 'on')
{		
    $line = <DATA>;
    chop($line);
    @SeriesNames = split( /\t/, $line);
    
    $numSeries = $#SeriesNames + 1;
}
else
{
    @dummy = split( /\t/, <DATA>);
    $numSeries = $#dummy + 1;
    for ($n = 0; $n < $numSeries; $n++)
    {
	# default unnamed series
	$SeriesNames[$n] = "Series ".$n;
    }
}
close (DATA);

############################## FORM GENERATING ############################

print $httpHeader;
$htmlBuf = &ReportHeader("Chart Options");

# main form tag
# call dispatcher without extra arguments, data on STDIN
$chartPath = $ENV{DISPATCHER_URL};
$chartPath .= "/chart";

$htmlBuf .= "<FORM METHOD=POST ACTION=\"$chartPath\">\n";
 
print $htmlBuf;

#######################  HIDDEN FIELDS ##############################

#bunch of hidden fields are just passed back to the cgi
$htmlBuf = &FormInputTag ("hidden", "setFile", $setFile);
$htmlBuf .= &FormInputTag ("hidden", "nameBase", $nameBase);
$htmlBuf .= &FormInputTag ("hidden", "reportTitle", $Settings{reportTitle});
$htmlBuf .= &FormInputTag ("hidden", "refSeries", $Settings{refSeries});
$htmlBuf .= &FormInputTag ("hidden", "hasSeriesLabels", 
			     $Settings{hasSeriesLabels});
$htmlBuf .= &FormInputTag ("hidden", "pieExplodeMax", "on" );

$htmlBuf .= &FormInputTag ("hidden", "dataFont", $Settings{dataFont});
$htmlBuf .= &FormInputTag ("hidden", "dataFontSize", 
			     $Settings{dataFontSize});
$htmlBuf .= &FormInputTag ("hidden", "dataFontStyle", $Settings{dataFontStyle});
$htmlBuf .= &FormInputTag ("hidden", "dataSymbol", $Settings{dataSymbol});

$htmlBuf .= &FormInputTag ("hidden", "chartShadows", 
			     $Settings{chartShadows});
$htmlBuf .= &FormInputTag ("hidden", "chart3D", $Settings{chart3D});
$htmlBuf .= &FormInputTag ("hidden", "dataPos", $Settings{dataPos});
if (defined ($Settings{lineSize}))
{
   $htmlBuf .= &FormInputTag ("hidden", "lineSize", $Settings{lineSize});
}

print $htmlBuf;

### center tables ###
$htmlBuf = "<CENTER>\n";
print $htmlBuf;

### Top submit button ###
$htmlBuf = "<TABLE BORDER=0 WIDTH=600>\n";
$htmlBuf .= "<TR>";
$htmlBuf .= "<TD><CENTER>\n";
$htmlBuf .= "<INPUT TYPE =\"submit\" VALUE=\"Make Chart\">";
$htmlBuf .= "</CENTER></TD>\n";
$htmlBuf .= "</TR>\n";
$htmlBuf .= "</TABLE>\n";
$htmlBuf .= "<BR>\n";

print $htmlBuf;


###  TABLE START ###

$htmlBuf = "<TABLE BORDER=2 WIDTH=\"600\" CELLSPACING=0>\n";
$htmlBuf .= "<TR>\n";
$htmlBuf .= "<TD VALIGN=TOP>\n";

print $htmlBuf;

###  Title ###

$htmlBuf = "<TR><TD COLSPAN=2><CENTER>\n";
$htmlBuf .= &Title(3, "Basic Chart Options");
$htmlBuf .= "\n</CENTER></TD></TR>\n";
print $htmlBuf;

####################### GLOBAL CHART PARAMETERS ###########################

$htmlBuf = "<TD VALIGN=TOP WIDTH=\"300\">\n";
$htmlBuf .= &Title (3, "Chart Type");

$curSel = $Settings{chartType};

$htmlBuf .= &FormInputTag ("radio", "chartType", 
			     $ChartTypes[2], 
			     $ChartTypes[2] eq $curSel ? 1:0);
$htmlBuf .= "Area &nbsp; &nbsp;";
$htmlBuf .= &FormInputTag ("radio", "chartType", 
			     $ChartTypes[0], 
			     $ChartTypes[0] eq $curSel ? 1:0);
$htmlBuf .= "Bar &nbsp; &nbsp;";
$htmlBuf .= &FormInputTag ("radio", "chartType", 
			     $ChartTypes[1], 
			     $ChartTypes[1] eq $curSel ? 1:0);
$htmlBuf .= "Line  &nbsp; &nbsp;\n"; 
$htmlBuf .= &FormInputTag ("radio", "chartType", 
			     $ChartTypes[4], 
			     $ChartTypes[4] eq $curSel ? 1:0);
$htmlBuf .= "Pie Chart &nbsp;\n";
$htmlBuf .= &FormInputTag ("radio", "chartType", 
			     $ChartTypes[3], 
			     $ChartTypes[3] eq $curSel ? 1:0);
$htmlBuf .= "Point</TD>\n";
print $htmlBuf;


#select chart orientation
$htmlBuf = "<TD VALIGN=TOP WIDTH=\"300\">\n";

$htmlBuf .= &Title (3, "Orientation");
$curSel = $Settings{chartOrientation};
$htmlBuf .= &FormInputTag ("radio", "chartOrientation", 
			     $ChartOrientations[0], 
			     $ChartOrientations[0] eq $curSel ? 1:0);
$htmlBuf .= "Horizontal\n";
$htmlBuf .= "&nbsp; &nbsp; &nbsp; &nbsp;";
$htmlBuf .= &FormInputTag ("radio", "chartOrientation", 
			     $ChartOrientations[1],
			     $ChartOrientations[1] eq $curSel ? 1:0);
$htmlBuf .= "Vertical\n";

$htmlBuf .= "</TD>\n";
$htmlBuf .= "</TR>\n";

print $htmlBuf;

# minimum dimensions
$htmlBuf = "<TR>\n";
$htmlBuf .= "<TD VALIGN=TOP WIDTH=\"300\">\n";
$htmlBuf .= &Title (3, "Dimensions");
# $htmlBuf .= &Title(5, "Chart will be no smaller than");
$htmlBuf .= "<PRE>";
$htmlBuf .= sprintf ("%10s ", "Width");
$htmlBuf .= &FormTextTag ("chartW", $Settings{chartW}, 5);
$htmlBuf .= sprintf (" minimum\n");
$htmlBuf .= sprintf ("\n%10s ", "Height");
$htmlBuf .= &FormTextTag ("chartH", $Settings{chartH}, 5);
$htmlBuf .= sprintf (" minimum");
$htmlBuf .= "</PRE>";
print $htmlBuf;

# series and colors
$htmlBuf = "</TD>\n";
$htmlBuf .= "<TD VALIGN=TOP WIDTH=\"300\">\n"; 
$htmlBuf .= &Title (3, "Colors");
# $htmlBuf .= &Title(5, "Select the series to chart and their colors");
$htmlBuf .= "<PRE>\n";
  
series:
for ($n = 0; $n < $numSeries; $n++)
{
    next series if ($n eq $Settings{refSeries});
    $formName = "chartSeries".$n; 
    $htmlBuf .= sprintf( "%10s ", $SeriesNames[$n]);
    $htmlBuf .= &FormInputTag ("checkbox", $formName, "on", 
				 $Settings{$formName} eq "on" ? 1: 0);
    $htmlBuf .= "    Color ";
    $formName = "colorSeries".$n;
    $htmlBuf .= &FormMenu ($formName, $Settings{$formName}, %Colors);
    $htmlBuf .= "&nbsp; &nbsp; &nbsp; \n";
}

$htmlBuf .= "</PRE>";
print $htmlBuf;

### end of easy settings 
$htmlBuf = "</TD></TR>\n";
$htmlBuf .= "</TABLE>\n";

print $htmlBuf;

#### table separator #######

$htmlBuf = "<P>";
$htmlBuf .= "<IMG SRC = \"$imgURL";
$htmlBuf .= "surf_ruleLITE.gif\">";
$htmlBuf .= "<P>";

print $htmlBuf;


### advanced settings ###

### new table
$htmlBuf = "<TABLE BORDER=2 WIDTH=\"600\" CELLSPACING=0>\n";
$htmlBuf .= "<TR>\n";
$htmlBuf .= "<TD COLSPAN=2><CENTER>\n";
$htmlBuf .= &Title(3, "Advanced Chart Features");
$htmlBuf .= "\n</CENTER></TD></TR>\n";
print $htmlBuf;

############################### TITLE BOX ####################################

#font parameters

$htmlBuf = "<TD VALIGN=TOP WIDTH=\"300\" >\n";
$htmlBuf .= &Title (3, "Titles");

$htmlBuf .= &FormInputTag ("hidden", "titleFont", $Settings{titleFont});
$htmlBuf .= "Title font size ";
$htmlBuf .= &FormTextTag ("titleFontSize", $Settings{titleFontSize}, 2);
$htmlBuf .= "&nbsp;&nbsp;&nbsp; &nbsp;\n";
$htmlBuf .= &FormInputTag ("hidden", "titleFontStyle", 
			     $Settings{titleFontStyle});

# frame & shadow
$htmlBuf .= &FormInputTag ("checkbox", "titleBox", "on", 
			    $Settings{titleBox} eq "on" ? 1 : 0);
$htmlBuf .= "Frame &nbsp; &nbsp;";
# $htmlBuf .= "Frame<BR><BR>\n";
$htmlBuf .= &FormInputTag ("checkbox", "titleBoxShadow", "on", 
			    $Settings{titleBoxShadow} eq "on" ? 1 : 0);
$htmlBuf .= "Shadow<BR>\n";

# position
$htmlBuf .= &Title (5, "Position");

$curSel = $Settings{titleBoxPos};
$htmlBuf .= "<PRE>"; 
#top row
$htmlBuf .= &FormInputTag ("radio" , "titleBoxPos", 
			     $BoxPositions[0], 
			     $BoxPositions[0] eq $curSel ? 1 : 0);
$htmlBuf .= "    ";
$htmlBuf .= &FormInputTag ("radio" , "titleBoxPos", 
			     $BoxPositions[1], 
			     $BoxPositions[1] eq $curSel ? 1 : 0);
$htmlBuf .= "    ";
$htmlBuf .= &FormInputTag ("radio" , "titleBoxPos", 
			     $BoxPositions[2], 
			     $BoxPositions[2] eq $curSel ? 1 : 0);
$htmlBuf .= "\n\n";
#middle row
$htmlBuf .= &FormInputTag ("radio" , "titleBoxPos", 
			     $BoxPositions[3], 
			     $BoxPositions[3] eq $curSel ? 1 : 0);
$htmlBuf .= "           ";
$htmlBuf .= &FormInputTag ("radio" , "titleBoxPos", 
			     $BoxPositions[4], 
			     $BoxPositions[4] eq $curSel ? 1 : 0);
$htmlBuf .= "\n\n";
#bottom row
$htmlBuf .= &FormInputTag ("radio" , "titleBoxPos", 
			     $BoxPositions[5], 
			     $BoxPositions[5] eq $curSel ? 1 : 0);
$htmlBuf .= "    ";
$htmlBuf .= &FormInputTag ("radio" , "titleBoxPos", 
			     $BoxPositions[6], 
			     $BoxPositions[6] eq $curSel ? 1 : 0);
$htmlBuf .= "    ";
$htmlBuf .= &FormInputTag ("radio" , "titleBoxPos", 
			     $BoxPositions[7], 
			     $BoxPositions[7] eq $curSel ? 1 : 0);
$htmlBuf .= "</PRE>\n";
$htmlBuf .= "</TD>\n";

print $htmlBuf;


############################### LABEL BOX #####################################

$htmlBuf = "<TD VALIGN=TOP WIDTH=\"300\">\n";
$htmlBuf .= &Title (3, "Label Box");

$htmlBuf .= &FormInputTag ("hidden", "labelFont", $Settings{labelFont});
$htmlBuf .= "Label font size ";
$htmlBuf .= &FormTextTag ("labelFontSize", $Settings{labelFontSize}, 2);
$htmlBuf .= &FormInputTag ("hidden", "labelFontStyle", 
			     $Settings{labelFontStyle});
$htmlBuf .= "&nbsp;&nbsp;&nbsp; &nbsp;";

# frame & shadow
$htmlBuf .= &FormInputTag ("checkbox", "labelBox", "on", 
			    $Settings{labelBox} eq "on" ? 1 : 0);
$htmlBuf .= "Frame  &nbsp; &nbsp;";
$htmlBuf .= &FormInputTag ("checkbox", "labelBoxShadow", "on", 
			    $Settings{labelBoxShadow} eq "on" ? 1 : 0);
$htmlBuf .= "Shadow<BR>\n";
$htmlBuf .= "(Label box is shown when more than one series is charted)<BR>"; 

# position
$htmlBuf .= &Title (5, "Position");

$curSel = $Settings{labelBoxPos};
$htmlBuf .= "<PRE>";
$htmlBuf .= &FormInputTag ("radio" , "labelBoxPos", 
			     $BoxPositions[0], 
			     $BoxPositions[0] eq $curSel ? 1 : 0);
$htmlBuf .= "    ";
$htmlBuf .= &FormInputTag ("radio" , "labelBoxPos", 
			     $BoxPositions[1], 
			     $BoxPositions[1] eq $curSel ? 1 : 0);
$htmlBuf .= "    ";
$htmlBuf .= &FormInputTag ("radio" , "labelBoxPos", 
			     $BoxPositions[2], 
			     $BoxPositions[2] eq $curSel ? 1 : 0);
$htmlBuf .= "\n\n";
$htmlBuf .= &FormInputTag ("radio" , "labelBoxPos", 
			     $BoxPositions[3], 
			     $BoxPositions[3] eq $curSel ? 1 : 0);
$htmlBuf .= "           ";
$htmlBuf .= &FormInputTag ("radio" , "labelBoxPos", 
			     $BoxPositions[4], 
			     $BoxPositions[4] eq $curSel ? 1 : 0);
$htmlBuf .= "\n\n";
$htmlBuf .= &FormInputTag ("radio" , "labelBoxPos", 
			     $BoxPositions[5], 
			     $BoxPositions[5] eq $curSel ? 1 : 0);
$htmlBuf .= "    ";
$htmlBuf .= &FormInputTag ("radio" , "labelBoxPos", 
			     $BoxPositions[6], 
			     $BoxPositions[6] eq $curSel ? 1 : 0);
$htmlBuf .= "    ";
$htmlBuf .= &FormInputTag ("radio" , "labelBoxPos", 
			     $BoxPositions[7], 
			     $BoxPositions[7] eq $curSel ? 1 : 0);
$htmlBuf .= "</PRE>\n";
$htmlBuf .= "</TD>\n";
$htmlBuf .= "</TR>\n";

print $htmlBuf;

############################ GLOBAL AXIS PARAMETERS ###########################

$htmlBuf = "<TR><TD COLSPAN=2>\n";
$htmlBuf .= &Title (3, "Axis");

#font
$htmlBuf .= &FormInputTag ("hidden", "axisLabelFont", 
			     $Settings{axisLabelFont});
$htmlBuf .= "Tick mark labels font size ";
$htmlBuf .= &FormTextTag ("axisLabelFontSize", $Settings{axisLabelFontSize}, 2);
$htmlBuf .= "&nbsp; &nbsp; &nbsp; &nbsp; &nbsp; \n";

$htmlBuf .= &FormInputTag ("hidden", "axisLabelFontStyle", 
			     $Settings{axisLabelFontStyle});

#ref axis direction
$htmlBuf .= &FormInputTag ("checkbox", "reverseRefDir", "on", 
			    $Settings{reverseRefDir} eq "on" ? 1 : 0);
$htmlBuf .= "Reverse main axis direction";
$htmlBuf .=  "</TD></TR>\n";

print $htmlBuf;


################################  X AXIS ################################

$htmlBuf = "<TR>\n";
$htmlBuf .= "<TD VALIGN=TOP WIDTH=\"300\">\n";
$htmlBuf .= &Title (5, "X Axis");

#tick position
$curSel = $Settings{xAxisTicks};


$htmlBuf .= sprintf ("<PRE>%25s", "Tick marks position ");
#$htmlBuf .= &FormMenu ($formName, $Settings{$formName}, %Colors);
#$htmlBuf .= "&nbsp; &nbsp; &nbsp; \n";


$htmlBuf .= "<SELECT NAME=\"xAxisTicks\">
             <OPTION VALUE=\"$Positions[0]\">outside
             <OPTION VALUE=\"$Positions[2]\" SELECTED>middle
             <OPTION VALUE=\"$Positions[1]\">inside
             </SELECT><BR>\n";
     



#$htmlBuf .= &FormInputTag ("radio" , "xAxisTicks", 
#			     $Positions[0], 
#			     $Positions[0] eq $curSel ? 1 : 0);
#$htmlBuf .= "outside  ";
#$htmlBuf .= &FormInputTag ("radio" , "xAxisTicks", 
#			     $Positions[2], 
#			     $Positions[2] eq $curSel ? 1 : 0);
#$htmlBuf .= "middle   ";
#$htmlBuf .= &FormInputTag ("radio" , "xAxisTicks", 
#			     $Positions[1], 
#			     $Positions[1] eq $curSel ? 1 : 0);
#$htmlBuf .= "inside   \n";

#labels
$htmlBuf .= sprintf ("\n%25s", "Show axis label ");
$htmlBuf .= &FormInputTag ("checkbox", "xAxisLabels", "on" ,
			     $Settings{xAxisLabels} eq "on" ? 1 : 0);
$htmlBuf .= sprintf ("\n\n%25s", "Show tick marks labels ");
$htmlBuf .= &FormInputTag ("checkbox", "xAxisScaleLabels", "on", 
			     $Settings{xAxisScaleLabels} eq "on" ? 1 : 0);
#grid lines
$htmlBuf .= sprintf ("\n\n%25s", "Show grid lines ");
$htmlBuf .= &FormInputTag ("checkbox", "xAxisGridLines", "on" ,
			     $Settings{xAxisGridLines} eq "on" ? 1 : 0);
$htmlBuf .= "</PRE>";
$htmlBuf .= "</TD>\n";

print $htmlBuf;

################################  Y AXIS ################################

#tick position

$htmlBuf = "<TD VALIGN=TOP WIDTH=\"300\">\n";
$htmlBuf .= &Title (5, "Y Axis");

$curSel = $Settings{yAxisTicks};
$htmlBuf .= sprintf ("<PRE>%25s", "Tick marks position ");
$htmlBuf .= "<SELECT NAME=\"yAxisTicks\">
             <OPTION VALUE=$Positions[0]>outside
             <OPTION VALUE=$Positions[2] SELECTED>middle
             <OPTION VALUE=$Positions[1]>inside
             </SELECT><BR>\n";
     


$htmlBuf .= sprintf ("\n%25s", "Show axis label ");
$htmlBuf .= &FormInputTag ("checkbox", "yAxisLabels", "on" ,
			     $Settings{yAxisLabels} eq "on" ? 1 : 0);
$htmlBuf .= sprintf ("\n\n%25s", "Show tick marks labels ");
$htmlBuf .= &FormInputTag ("checkbox", "yAxisScaleLabels", "on", 
			     $Settings{yAxisScaleLabels} eq "on" ? 1 : 0);
#grid lines
$htmlBuf .= sprintf ("\n\n%25s", "Show grid lines ");
$htmlBuf .= &FormInputTag ("checkbox", "yAxisGridLines", "on" ,
			     $Settings{yAxisGridLines} eq "on" ? 1 : 0);

$htmlBuf .= "</PRE><P>\n";
$htmlBuf .= "</TD></TR>\n";
$htmlBuf .="</TABLE>\n";

print $htmlBuf;



### Bottom submit button ###
$htmlBuf = "<P";
$htmlBuf .= "<TABLE BORDER=0 WIDTH=600>\n";
$htmlBuf .= "<TR>";
$htmlBuf .= "<TD><CENTER>\n";
$htmlBuf .= "<INPUT TYPE =\"submit\" VALUE=\"Make Chart\">";
$htmlBuf .= "</CENTER></TD>\n";
$htmlBuf .= "</TR>\n";
$htmlBuf .= "</TABLE>\n";
$htmlBuf .= "<BR>\n";

print $htmlBuf;

### end this form ###
$htmlBuf =  "</CENTER>\n";
$htmlBuf .= &InterfaceLink();
$htmlBuf .=  "<H5>SurfReport 2.1.7 <br>Copyright &copy; 1996, 1997 ";
$htmlBuf .= "<A HREF = \"http://www.bienlogic.com\">Bien Logic, Inc.</A></H5>";
$htmlBuf .= "</BODY></HTML>";

print $htmlBuf;

#################################  SUBROUTINES  ###############################

sub Title
{
    local ($size, $text) = @_;
    local ($buf);
    $buf = "\n<H$size>$text</H$size>\n";
}

sub FormInputTag
{
    local ($type, $name, $value, $checked) = @_;
    local ($buf);
    $buf = "<INPUT TYPE=\"$type\" NAME=\"$name\" VALUE=\"$value\"";
    if (defined ($checked) && $checked == 1)
    {
	$buf .= " CHECKED";
    }
    $buf .= ">";
}

sub FormTextTag
{
    local ($name, $value, $size) = @_;
    local ($buf);

    $buf = "<INPUT NAME=\"$name\" VALUE=\"$value\" SIZE=\"$size\"";
    $buf .= ">";
}

sub FormMenu
{
    local ($name, $selected, %options) = @_;
    local ($buf, $n, $value);
    
    $buf = "<SELECT NAME=\"$name\">\n";
    
    foreach $name (sort(keys %options))
    {
	$buf .= "<OPTION VALUE=\"";
	$value = $options{$name};
	$buf .= $value."\"";
	if ($value eq $selected)
	{
	    $buf .= " SELECTED";
	}
	$buf .= ">$name\n";
    }
    $buf .= "</SELECT>";
     
}

sub FormRadio
{
    local ($name, $selected, @list) = @_;
    local ($buf, $n);
    
    $buf = "\n<UL>\n";
    for ($n = 0; $n < $#list+1; $n++)
    {
	$buf .= "<BR><INPUT TYPE=\"radio\" NAME=\"$name\" ";
	$buf .= "VALUE=\"$list[$n]\"";
	if ($n == $selected)
	{
	    $buf .= " CHECKED";
	}
	$buf .= ">$list[$n]";
    }
    $buf .= "</UL>\n";
  
}


sub PostTextError
{
    local ($errorMsg, $sysError) = @_;

    print "Sorry, SurfReport Found an Error:\n";
    print "$errorMsg\n";
    if( $sysError ne '')
    {
	print "The system reported this error:\n";
	print "$sysError\n";
    }
    print "Please correct and try again!\n";
    print "\n";
    
}

#
# display an error message on the web, without using the config file
#
sub FatalError
{
    local ($err) = @_;
    
    if ($standAlone == 1)
    {
    	&PostTextError ($err);
    	return;
    }
    
    print "<body BGCOLOR = #FFFFFF TEXT = #OOOOOO >\n";
    print "<H3>Chart Design Error</H3>\n";
    

    print "<pre>\n\n";
    print $err;
    print "\n\n</pre>";

    print &PrintHTMLEnd();
}


#
# print end of page credits 
#
sub PrintHTMLEnd
{
	local ($htmlBuf);
	
    $htmlBuf = "<H5>SurfReport 2.1.7<br>";
	$htmlBuf .= "Copyright &copy; 1996, 1997 ";
	$htmlBuf .= "<A HREF = \"http://www.bienlogic.com\">";
	$htmlBuf .= "Bien Logic, Inc.</A></H5>\n";
	$htmlBuf .= "</BODY></HTML>";
	
	return $htmlBuf;
}

sub Error
{
    
    local ($errorMsg, $sysError) = @_;
    if ($standAlone == 1)
    {
		&PostTextError($errorMsg, $sysError);
    }
    else
    {		
    	$errorMsg .= "\n";
    	if (defined ( $sysError))
    	{
    		$errorMsg .= "The system reported this error: ";
    		$errorMsg .= $sysError;		
    	}
	&PostError($errorMsg, 1);
    }
}


# get setting file into an assoc. array
# settings files follow the same format as FORM 
# output : name=value&
sub GetSettings
{
    local ($settingsFile) = @_;
    local (@pairs, $buffer, $name, $value, $len);
 
    if (!open (SETTINGS, "$settingsFile"))
    {
	&Error ("Can't open the settings file '$settingsFile'", $!);
	exit;
    }
    seek (SETTINGS, 0, 2);
    $len = tell( SETTINGS);
    seek (SETTINGS, 0, 0);
    read(SETTINGS, $buffer, $len);
    @pairs = split( /&/, $buffer);

    foreach $pair (@pairs)
    {
	($name, $value) = split(/=/, $pair);
	$Settings{$name} = $value;
    }

    close (SETTINGS);
    
}


