#!/usr/bin/perl


################################ SRREPORT.CGI #################################
#
# V 2.1.7 Single User
#
# utility to translate tab delimited files created by the core app into legible
# text (for email and stand alone) and neatly formatted HTML docs
#
# The name of the settings file is passed as first argument
# This is the last use of this file in the sr2 chain so
# it gets deleted
#
# Author : Michel G. Delory
# created: March 1996
# 
# last rev Dec 12 1997
#
# copyright BienLogic Inc.
#
########################### ARRAYS & STRINGS ################################

$httpHeader = "Content-type: text/html\n\n";
$textLine = "\n                     ------------------------------\n\n";
$htmlLine = "<P>\n<HR WIDTH = 80%>\n";  

$htmlTitleH = "<P><FONT SIZE = +2>";
$htmlTitleT = "</FONT><BR><PRE>\n";
$htmlEndReport = "</PRE>".$htmlLine;

######################### ENVIRONMENT ######################

if ($ENV{STAND_ALONE} eq "YES")
{
    $standAlone = 1;
}
else
{
    $standAlone = 0;
}

$settingsFile = $ARGV[0];


if (!defined ($ENV{SURFREPORT_CONFIG})||
    !defined ($ENV{SURFREPORT_LIB}))
{
    #V2.0B7
    #can't acces config file, forget this

    $err = "SurfReport environment is not available.\n";
    $err .= "Can't access your configuration file\n";
    $err .= "You must run this script through the SurfReport dispatcher\n";

    &Error ($err);

    exit (-1);

}

require $ENV{SURFREPORT_CONFIG};
require $ENV{SURFREPORT_LIB};


#get the base path for all SR output files
#and make sure all needed env vars are available
if (!defined ($platform = $ENV{PLATFORM}) ||
    !defined ($surfOut = $ENV{SURFREPORT_OUT}) ||
    !defined ($ENV{SURFREPORT_WEBHOME}) ||
    !defined ($ENV{DISPATCHER_URL}) ||
    !defined ($mailService = $ENV{MAIL_SERVICE}))
{
    # exit now if env vars not available
    $err = "SurfReport environment is not available.\n";
    $err .= "You must run this script through the SurfReport dispatcher\n";

    &Error ($err);

    exit (-1);

}



# get a file path from the root web server
# used to open html file to fetch their title
$rootPath = $ENV{SURFREPORT_DOCROOT};



############################### FILES SETUP #################################

&GetSettings();

#get base URL for the chart icons
$chartImgURL = $imgURL."surf_chart";

#get nameBase for renaming output files
#output files get transfered from the <tmp> folder
#to the <SURFREPORT_OUT> folder
$nameBase = $Settings{nameBase};

$dataFileName = $surfOut . $nameBase . ".txt";
$dataTmpFile = $settingsFile .".txt";

unless (open (DATA, "$dataTmpFile"))
{
    &Error ("Can't open data file '$dataTmpFile'.", $!);
    exit;
}

$tocFileName = $surfOut . $nameBase . ".toc";
$tocTmpFile = $settingsFile . ".toc";

unless (open (TOC, "$tocTmpFile"))
{
    &Error ("Can't open table of content file '$tocTmpFile'.", $!);
    exit;
}


$email = $Settings{deliverMail} eq '1' ? 1 : 0;
$emailTab = $Settings{emailTab} eq '1' ? 1: 0;
$html = $Settings{deliverWeb} eq '1' ? 1 : 0;
if (defined ($ENV{SURFREPORT_DIRECTOUTPUT}))
{
    $screen = 0;
}
else
{
    $screen = $standAlone;
}
$htmlOnly = $Settings{htmlOnly};
$htmlTitles = $Settings{docTitles};

if ($screen == 1)
{
    $text = 1;
}


if ($email == 1 || $emailTab == 1)
{
    $text = 1;
    $emailTo = $Settings{emailTo}; 
    # change format from newline separated to comma separated
    $emailTo =~ s/\n/,/g;
} 


if ($email == 1)
{
	if ($platform eq "OS_UNIX")
	{   
   		#pipe EMAIL printing to mailer program  
   		if (!open(EMAIL, "| $mailService -t"))
   		{
       		&Error ("Can't pipe to '$mailService'\n", $!);
       		exit;
   		}
   		print EMAIL "To: $emailTo\n";
   		print EMAIL "Subject: SurfReport statistics\n\n";
	}
	elsif ($platform eq "OS_NT")
	{
   		$mailTmpFile = $settingsFile .".mail";
   		if (!open(EMAIL, ">$mailTmpFile"))
   		{
       		&Error ("Can't open  '$mailTmpFile'\n", $!);
       		exit;
   		}
	}
}


if ($html == 1)
{
    # create tmp file name for html output
    $outputFileName = $surfOut . $nameBase . ".html";
    $outputTmpFile = $settingsFile. ".html";

    if (!open (OUT, ">$outputTmpFile")) #changed from time
    {
	&Error ("Can't open output file '$outputTmpFile'.", $!);
	exit;
    }
    else
    {
	chmod (0666, $outputTmpFile);
    }
}

# call the dispatcher who will call the charting application
#parameters to the app are
#nameBase and set (.bl) file
if ($standAlone == 1)
{
    # when running stand alone, we don't know the URL to the dispatcher
    # so we rely on the URL saved in the config file 
    $chartURL = $dispatcherURL;
}
else
{
    $chartURL = $ENV{DISPATCHER_URL};
}

$chartURL .= "/chart/$nameBase";


############################### PROCESSING #################################

# build an assoc. array out of the Table Of Contents 
# needed to determine which reports were generated
# to include proper links
while(<TOC>)
{
    #tab delimited file of format
    #<report title><tab><position in the data file><newline>
    ($title, $pos) = split (/\t/);
    $Toc{$title} = $pos;

}
close (TOC);

#build links pointing to different parts of the doc
#the number of links depends on what reports were generated 
$links = "<BR>\n";

if (defined ($Toc{$TocNames[0]}))
{
    # report title, this may go
    #$links .= "<A HREF = \"#top\">[Top]</A>\n";
}

if (defined ($Toc{$TocNames[1]}) || 
    defined ($Toc{$TocNames[2]}))
{
    # totals and averages
    $links .= "<A HREF = \"#totals\">[Totals]</A>\n";
}
if (defined ($Toc{$TocNames[3]}) || 
    defined ($Toc{$TocNames[4]}))
{
    # totals and averages
    $links .= "<A HREF = \"#averages\">[Averages]</A>\n";
}
if (defined ($Toc{$TocNames[5]}) || 
    defined ($Toc{$TocNames[7]}) || 
    defined ($Toc{$TocNames[8]}) ||
    defined ($Toc{$TocNames[9]}) ||
    defined ($Toc{$TocNames[11]}))
{
    # Visitor Info
    $links .= "<A HREF = \"#visitors\">[Visitors]</A>\n";
}
if (defined ($Toc{$TocNames[6]}) || defined ($Toc{$TocNames[10]}))
{
    # Files Info
    $links .= "<A HREF = \"#files\">[Files]</A>\n";
}
if (defined ($Toc{$TocNames[17]}) || defined ($Toc{$TocNames[18]}))
{
    # Files Info
    $links .= "<A HREF = \"#transfers\">[Transfer]</A>\n";
}

if (defined ($Toc{$TocNames[12]}) || 
    defined ($Toc{$TocNames[13]}) ||
    defined ($Toc{$TocNames[15]}) ||
    defined ($Toc{$TocNames[16]}) ||
    defined ($Toc{$TocNames[14]}) ||
    defined ($Toc{$TocNames[19]}))
{
    # the rest of it
    $links .= "<A HREF = \"#other\">[Other Info]</A>\n";
}

$links .= "<BR>\n";

if ($html == 1)
{
    if ($standAlone != 1)
    {
	print $httpHeader;
    }
    $htmlBuf = &ReportHeader();
    
    # insert [Top] link, may go...
    #$htmlBuf .= "<A NAME =top></A>";
}

if ($text == 1)
{
    $textBuf = sprintf ("\n%46s$textLine","SurfReport Statistics");
}
&Output( $htmlBuf, $textBuf);

report: 
for ($i = 0; $i <= $#TocNames; $i++) 
{ 
    $title = $TocNames[$ReportSeq[$i]-1]; 
    # that particular report was unchecked in the html form 
    next report if (!defined ($Toc{$title}));

    #position the data file 
    seek (DATA, int($Toc{$title}), 0);
    &Report ($title);
}

# close the report with registration info
# and link to BienLogic
if ($html == 1)
{
    $htmlBuf = $links. "<BR>";
    $htmlBuf .= &InterfaceLink();
    $htmlBuf .= "<P>";

    $htmlBuf .= "<PRE>";
    if ($#reg == 0)
    {
	# Unregistered copy
	$htmlBuf .= $reg[0];
    }
    else
    {
	# owner
	$htmlBuf .= $reg[0]." <B>".$reg[1]."</B>\n";
	#partial serial number
	$htmlBuf .= $reg[2]." <B>".$reg[3]."</B>\n";
    }
    $htmlBuf .= "</PRE><P>";
    $htmlBuf .= "<H5>SurfReport 2.1.7<br>";
    $htmlBuf .= "Copyright &copy; 1996, 1997 ";
    $htmlBuf .= "<A HREF = \"http://www.bienlogic.com\">";
    $htmlBuf .= "Bien Logic, Inc.</A></H5>\n";
    
    $htmlBuf .= "</BODY></HTML>";
}
if( $text == 1)
{
    if ($#reg == 0)
    {
	# unregistered copy
	$textBuf = $reg[0]."\n";
    }
    else
    {
	# owner
	$textBuf = $reg[0]." ".$reg[1]."\n";
	#partial serial number
	$textBuf .= $reg[2]." ".$reg[3]."\n";
    }

    $textBuf .= "SurfReport 2.1.7\n";
    $textBuf .= "Copyright (c); 1996, 1997 Bien Logic, Inc\n";
   
}
&Output ($htmlBuf, $textBuf);

# send mail report now
if ($email == 1)
{
	close (EMAIL);
   	
   	if ($platform eq "OS_NT")
   	{
      system("$mailService $mailTmpFile -t $emailTo -s \"SurfReport statistics\"");
      unlink ($mailTmpFile);  
   	}
}
# email tab file 
if ($emailTab == 1)
{
   # email the tab delimited file
   if ($platform eq "OS_UNIX")
   {
      if (!open(EMAIL_TAB, "| $mailService -t"))
      {
          &Error ("Can't pipe to '$mailService'", $!);
         exit;
      }
      print EMAIL_TAB "To: $emailTo\n";
      print EMAIL_TAB "Subject: SurfReport Tab File\n";
   }
   elsif ($platform eq "OS_NT")
   {
      $tabTmpFile = $settingsFile .".tab";
      if (!open(EMAIL_TAB, ">$tabTmpFile"))
      {
          &Error ("Can't open '$tabTmpFile'", $!);
          exit;
      }
   }
   # rewind data file 
   seek( DATA, 0, 0);
   while ($line = <DATA>)
   {
       print EMAIL_TAB $line;
   }
   print EMAIL_TAB "SurfReport 2.1.7\n";
   print EMAIL_TAB "Copyright (c); 1996, 1997 Bien Logic, Inc\n";   
   
   #UNIX will pipe through, NT will just close the mail file
   close (EMAIL_TAB);
   
   if ($platform eq "OS_NT")
   {
      system("$mailService $tabTmpFile -t $emailTo -s \"SurfReport Tab File\"");
      unlink ($tabTmpFile);
   }

}


close (DATA);

if ($html == 1)
{
    close (OUT);
    rename ($outputTmpFile, $outputFileName); #changed from $time
}

# get rid of the temporary settings file
# this will also indicate the report 
# is ready to be displayed

unlink ($settingsFile);
rename ($dataTmpFile, $dataFileName);
rename ($tocTmpFile, $tocFileName);

if (defined ($ENV{SURFREPORT_DIRECTOUTPUT}))
{
    # does not go through srcheck.cgi
    unlink <$settingsFile*>;
}


############################## SUBROUTINES ##################################

sub Report
{
    local ($title) = @_;
   
    if ($title eq $TocNames[0])
    {
	&ReportTitle();
    }
    # Daily Total
    # Period Totals
    # Daily Avg
    # Hourly Avg
    elsif ($title eq $TocNames[1])
    {
	if (! defined ($ltotal))
	{
	    
	    &Output ("<A NAME=totals></A>", "");
	    &Output ($links, "");
	    $ltotal = 1;
	}
 	&DailyTotal();
    }
    elsif ($title eq $TocNames[2])
    {
	if (! defined ($ltotal))
	{

	    &Output ("<A NAME=totals></A>", "");
	    &Output ($links, "");
	    $ltotal = 1;
	}
	&PeriodTotal();
    }
    elsif ($title eq $TocNames[3])
    {
	if (!defined ($lavg))
	{
	    
	    &Output ("<A NAME=averages></A>", "");
	    &Output ($links, "");
	    $lavg = 1;
	}
	&DailyAvg();
    }
    elsif ($title eq $TocNames[4])
    {
	if (!defined ($lavg))
	{
	    
	    &Output ("<A NAME=averages></A>", "");
	    &Output ($links, "");
	    $lavg = 1;
	}
	&HourlyAvg();
    }
    # Top Visitors
    # Top Domains (companies)
    # Top Networks (countries)
    # Top Referrers 
    # Particular Files (gives a list of visitors)
    elsif ($title eq $TocNames[5])
    {
	if (!defined ($lvisit))
	{

	    &Output ("<A NAME=visitors></A>", "");
	    &Output ($links, "");
	    $lvisit = 1;
	}
	&TopClientList();
    }
    elsif ($title eq $TocNames[7])
    {
	if (!defined ($lvisit))
	{

	    &Output ("<A NAME=visitors></A>", "");
	    &Output ($links, "");
	    $lvisit = 1;
	}
	# top domain
	&TopExtendedList();
    }
    elsif ($title eq $TocNames[8])
    {
	if (!defined ($lvisit))
	{

	    &Output ("<A NAME=visitors></A>", "");
	    &Output ($links, "");
	    $lvisit = 1;
	}
	# top nets
	&TopExtendedList();
    }
    elsif ($title eq $TocNames[9])
    {
	if (!defined ($lvisit))
	{

	    &Output ("<A NAME=visitors></A>", "");
	    &Output ($links, "");
	    $lvisit = 1;
	}
	# top referrers
	&TopList();
    }
    elsif ($title eq $TocNames[11])
    {
	if (!defined ($lvisit))
	{

	    &Output ("<A NAME=visitors></A>", "");
	    &Output ($links, "");
	    $lvisit = 1;
	}
	# particular files
	&MatchList();
      
    }
    # Top Files
    # Particular Visitor ( gives a list of Files )
    elsif ($title eq $TocNames[6])
    {
	if (!defined ($lfile))
	{

	    &Output ("<A NAME=files></A>", "");
	    &Output ($links, "");
	    $lfile = 1;
	}
	&TopFilesList();
    }
    elsif ($title eq $TocNames[10])
    {
	if (!defined ($lfile))
	{

	    &Output ("<A NAME=files></A>", "");
	    &Output ($links, "");
	    $lfile = 1;
	}
	&MatchClientsList();
	
    }
    # Excluded Visitors
    # Excluded Files
    # Browsers
    # Platforms
    # Http Errors
    # File Processed
    elsif ($title eq $TocNames[12])
    {
	if (!defined ($lother))
	{

	    &Output ("<A NAME=other></A>", "");
	    &Output ($links, "");
	    $lother = 1;
	}
	# excluded visitors
	&XReport();
    }
    elsif ($title eq $TocNames[13])
    {
	if (!defined ($lother))
	{

	    &Output ("<A NAME=other></A>", "");
	    &Output ($links, "");
	    $lother = 1;
	}
	# excluded files
	&XReport();

    }
    elsif ($title eq $TocNames[15])
    {
	if (!defined ($lother))
	{

	    &Output ("<A NAME=other></A>", "");
	    &Output ($links, "");
	    $lother = 1;
	}
	&Browsers("brwall.bl");
    }
    elsif ($title eq $TocNames[16])
    {
	
	if (!defined ($lother))
	{

	    &Output ("<A NAME=other></A>", "");
	    &Output ($links, "");
	    $lother = 1;
	}
	# Operating Systems (same format as browsers)
        &Browsers("pltall.bl");
    }
    elsif ($title eq $TocNames[14])
    {
	if (!defined ($lother))
	{

	    &Output ("<A NAME=other></A>", "");
	    &Output ($links, "");
	    $lother = 1;
	}
	# http errors
	&Status();
    }
    elsif ($title eq $TocNames[19])
    {
	if (!defined ($lother))
	{

	    &Output ("<A NAME=other></A>", "");
	    &Output ($links, "");
	    $lother = 1;
	}
	# processed log files
	&Process();
    }
    elsif ($title eq $TocNames[17])
    {
		if (! defined ($ltransfer))
		{
	    	
	    	&Output ("<A NAME=transfers></A>", "");
	    	&Output ($links, "");
	    	$ltransfer = 1;
		}
 		&DailyTransfer();
    }
    elsif ($title eq $TocNames[18])
    {
		if (! defined ($ltransfer))
		{
	    	
	    	&Output ("<A NAME=transfers></A>", "");
	    	&Output ($links, "");
	    	$ltransfer = 1;
		}
 		&HourlyTransfer();
    }
    else
    {
	&Error ("Unknown Report type '$title'");
	exit;
    }
}

sub DailyTotal
{
    local ($title, @data);
    local ($textBuf, $htmlBuf, $buf);

    $textBuf = $htmlBuf = "";

    $title = <DATA>;
    if ($html == 1)
    {
	$htmlBuf = $htmlTitleH.$title.$htmlTitleT;
    }
    if ($text == 1)
    {				
	$textBuf = $title."\n";
    }
    
    @data = split( /\t/, <DATA>);
    $buf = sprintf("%s%30s%12s%13s\n", 
		      $data[0], $data[1], $data[2], $data[3]);
 
  getData:
    while ($nextLine = <DATA>)
    {
	last getData if ($nextLine eq "\n");
	    
	@data = split (/\t/, $nextLine);
	$buf .= sprintf( "%15s%19d%12d%12d\n",
			  $data[0], $data[1], $data[2], $data[3]);
       
    }
    if ($html == 1)
    {
	$htmlBuf .= $buf;
	$htmlBuf .= "\n";

	$htmlBuf .= "<B>Chart:</B> <A HREF=\"$chartURL/dtall.bl\">";
	$htmlBuf .= "<IMG BORDER=0 SRC=\"$chartImgURL"."all.gif\"";
	$htmlBuf .= " ALT = Chart All ALIGN = MIDDLE></A>";

	$htmlBuf .= "<A HREF=\"$chartURL/dtvis.bl\">";
	$htmlBuf .= "<IMG BORDER=0 SRC=\"$chartImgURL"."visits.gif\"";
	$htmlBuf .= " ALT = Chart Visitors ALIGN = MIDDLE></A>&nbsp;";

	$htmlBuf .= "<A HREF=\"$chartURL/dtpage.bl\">";
	$htmlBuf .= "<IMG BORDER=0 SRC=\"$chartImgURL"."pages.gif\"";
	$htmlBuf .= " ALT = Chart Pages ALIGN = MIDDLE></A>&nbsp;";

	$htmlBuf .= "<A HREF=\"$chartURL/dthit.bl\">";
	$htmlBuf .= "<IMG BORDER=0 SRC=\"$chartImgURL"."hits.gif\"";
	$htmlBuf .= " ALT = Chart Hits ALIGN = MIDDLE></A>&nbsp;";
	
	$htmlBuf .= "<br>";
	
	$htmlBuf .= $htmlEndReport;
    }
    if ($text == 1)
    {
	$textBuf .= $buf.$textLine;
    }
    &Output ($htmlBuf, $textBuf);
}

sub PeriodTotal
{
    local ($title, @data);
    local ($textBuf, $htmlBuf, $buf);

    $textBuf = $htmlBuf = "";

    $title = <DATA>;
    if ($html == 1)
    {
	$htmlBuf = $htmlTitleH.$title.$htmlTitleT;
    }
    if ($text == 1)
    {				
	$textBuf = $title."\n";
    }
    
    
    @data = split( /\t/, <DATA>);
    $buf .= sprintf("%18s%16s%12s%13s\n", 
		    $data[0], $data[1], $data[2], $data[3]);
    @data = split( /\t/, <DATA>);
    $buf .= sprintf("%18d%16d%12d%12d\n", 
		    $data[0], $data[1], $data[2], $data[3]);
    
    if ($html == 1)
    {
	$htmlBuf .= $buf.$htmlEndReport;
    }
    if ($text == 1)
    {
	$textBuf .= $buf.$textLine;
    }
    &Output ($htmlBuf, $textBuf);
}

sub DailyAvg
{				
    local ($title, @data);
    local ($textBuf, $htmlBuf, $buf);

    $textBuf = $htmlBuf = "";

    $title = <DATA>;
    if ($html == 1)
    {
	$htmlBuf = $htmlTitleH.$title.$htmlTitleT;
    }
    if ($text == 1)
    {				
	$textBuf = $title."\n";
    }
    
    @data = split( /\t/, <DATA>);
    $buf .= sprintf("%18s%16s%12s%13s\n", 
		    $data[0], $data[1], $data[2], $data[3]);
    @data = split( /\t/, <DATA>);
    $buf .= sprintf("%18d%16d%12d%12d\n", 
		    $data[0], $data[1], $data[2], $data[3]);
    
    if ($html == 1)
    {
	$htmlBuf .= $buf.$htmlEndReport;
    }
    if ($text == 1)
    {
	$textBuf .= $buf.$textLine;
    }
    &Output ($htmlBuf, $textBuf);
}

sub HourlyAvg
{
    local ($title, @data);
    local ($textBuf, $htmlBuf, $buf);

    $textBuf = $htmlBuf = "";

    $title = <DATA>;
    if ($html == 1)
    {
	$htmlBuf = $htmlTitleH.$title.$htmlTitleT;
    }
    if ($text == 1)
    {				
	$textBuf = $title."\n";
    }
    
    @data = split( /\t/, <DATA>);
    $buf .= sprintf("%31s%15s%15s\n", $data[0], $data[1], $data[2]);
 
  getData:
    while ($nextLine = <DATA>)
    {
	last getData if ($nextLine eq "\n");
	    
	@data = split (/\t/, $nextLine);
	$buf .= sprintf( "%34s%12d%12.1f %s\n",
			$data[0], $data[1], $data[2], "%");
       
    }

    if ($html == 1)
    {
	$htmlBuf .= $buf;
	$htmlBuf .= "\n";

	$htmlBuf .= "<b>Chart:</b> <A HREF=\"$chartURL/haall.bl\">";
	$htmlBuf .= "<IMG BORDER=0 SRC=\"$chartImgURL"."hours.gif\"";
	$htmlBuf .= " ALT = Chart Hours ALIGN = MIDDLE></A>&nbsp;";

	$htmlBuf .= $htmlEndReport;
    }
    if ($text == 1)
    {
	$textBuf .= $buf.$textLine;
    }
    &Output ($htmlBuf, $textBuf);
}
sub TopClientList
{
    local ($title, @data);
    local ($textBuf, $htmlBuf, $buf);

    $textBuf = $htmlBuf = "";

    @data = split (/\t/, <DATA>);
    $title = $data[0]."\n";

    # header
    @data = split( /\t/, <DATA>);

	if ($data[0] =~ /\S+/)
    {
	$buf = sprintf("%6s%8s   %s\n", $data[0], $data[1], $data[2]);
    }
    else
    {
	# header line is made out of spaces (placeholders) if 
        # there is nothing in the report -> no header 
	$buf = "";
    }
    
    if ($html == 1)
    {
	$htmlBuf = $htmlTitleH.$title.$htmlTitleT.$buf;
    }
    if ($text == 1)
    {				
	$textBuf = $title."\n".$buf;
    }

    
  getData:
    while ($nextLine = <DATA>)
    {
	last getData if ($nextLine eq "\n");
	    
	@data = split (/\t/, $nextLine);

$client = $data[2];

if (( $fResolveIP == 1) && (int( $client) != 0))
        {
            #try to resolve ip address
            $hostName = &ResolveIP( $client);

            if ( defined ( $hostName))
            {
                $data[2] = $hostName."\n";
            }
        }

	if ($html == 1)
	{
	    $htmlBuf .= sprintf ("<B>%6d%8d</B>   %s",
				 $data[0], $data[1], $data[2]);
	}
	if ($text == 1)
	{
	    $textBuf .= sprintf ("%6d%8d   %s", 
				 $data[0], $data[1], $data[2]);
	}
	
    }
    if (($nextLine = <DATA>) ne "\n")
    {
	# comment after the list ... 
	@data = split (/\t/, $nextLine);
	$buf = sprintf ("          $data[0]");
	$htmlBuf .= $buf;
	$textBuf .= $buf;
       
    }
    if ($html == 1)
    {
	$htmlBuf .= "\n";

	$htmlBuf .= "<b>Chart:</b> <A HREF=\"$chartURL/tcall.bl\">";
	$htmlBuf .= "<IMG BORDER=0 SRC=\"$chartImgURL"."visitors.gif\"";
	$htmlBuf .= " ALT = Chart Visitors ALIGN = MIDDLE></A>&nbsp;";

	$htmlBuf .= $htmlEndReport;
       
    }
    if ($text == 1)
    {
	$textBuf .= $textLine;
    }
    &Output ($htmlBuf, $textBuf);
}

sub TopList
{
    local ($title, @data);
    local ($textBuf, $htmlBuf, $buf);

    $textBuf = $htmlBuf = "";

    @data = split (/\t/, <DATA>);
    $title = $data[0]."\n";

    if ($html == 1)
    {
	$htmlBuf = $htmlTitleH.$title.$htmlTitleT;
    }
    if ($text == 1)
    {				
	$textBuf = $title."\n";
    }
  
  getData:
    while ($nextLine = <DATA>)
    {
	last getData if ($nextLine eq "\n");
	    
	@data = split (/\t/, $nextLine);
	
	if ($html == 1)
	{
	    $htmlBuf .= sprintf ("<B>%6d    </B>%s", $data[0], $data[1]);
	}
	if ($text == 1)
	{
	    $textBuf .= sprintf ("%6d    %s", $data[0], $data[1]);
	}
	
    }
    if (($nextLine = <DATA>) ne "\n")
    {
	# comment after the list ... 
	@data = split (/\t/, $nextLine);
	$buf = sprintf ("          $data[0]");
	$htmlBuf .= $buf;
	$textBuf .= $buf;
       
    }
    if ($html == 1)
    {
	$htmlBuf .= $htmlEndReport;
    }
    if ($text == 1)
    {
	$textBuf .= $textLine;
    }
    &Output ($htmlBuf, $textBuf);
}

sub TopExtendedList
{
    local ($title, @data);
    local ($textBuf, $htmlBuf, $buf);

    $textBuf = $htmlBuf = "";

    @data = split (/\t/, <DATA>);
    $title = $data[0]."\n";
    
    @data = split( /\t/, <DATA>);
    if ($data[0] =~ /\S+/)
    {

	if (!defined ($data[3]) || $data[3] eq "")
	{
	    $buf = sprintf("%6s%8s   %s\n", $data[0], $data[1], $data[2]);
	}
	else
	{
	    $buf = sprintf("%6s%8s   %-35s%s\n", $data[0], $data[1], $data[2], $data[3]);
	}
    }
    else
    {
    	$buf = "";
	}
	
    if ($html == 1)
    {
	$htmlBuf = $htmlTitleH.$title.$htmlTitleT.$buf;
    }
    if ($text == 1)
    {				
	$textBuf = $title."\n".$buf;
    }
  
  getData:
    while ($nextLine = <DATA>)
    {
	last getData if ($nextLine eq "\n");
	    
	@data = split (/\t/, $nextLine);
	if (!defined ($data[3]) || $data[3] eq "")
	{
	 
	    if ($html == 1)
	    {
		$htmlBuf .= sprintf ("<B>%6d%8d</B>   %s", $data[0], $data[1], $data[2]);
	    }
	    if ($text == 1)
	    {
		$textBuf .= sprintf ("%6d%8d    %s", $data[0], $data[1], $data[2]);
	    }
	}
	else
	{
	    if ($html == 1)
	    {
		$htmlBuf .= sprintf ("<B>%6d%8d</B>   %-35s%s", $data[0], $data[1], $data[2], $data[3]);
	    }
	    if ($text == 1)
	    {
		$textBuf .= sprintf ("%6d%8d    %-35s%s", $data[0], $data[1], $data[2], $data[3]);
	    }
	}
	
    }
    if (($nextLine = <DATA>) ne "\n")
    {
	# comment after the list ... 
	@data = split (/\t/, $nextLine);
	$buf = sprintf ("          $data[0]");
	$htmlBuf .= $buf;
	$textBuf .= $buf;
       
    }
    if ($html == 1)
    {
	$htmlBuf .= $htmlEndReport;
    }
    if ($text == 1)
    {
	$textBuf .= $textLine;
    }
    &Output ($htmlBuf, $textBuf);
}

sub TopFilesList
{
    local ($title, @data);
    local ($textBuf, $htmlBuf, $buf);

    $textBuf = $htmlBuf = "";
	@data = split (/\t/, <DATA>);
    $title = $data[0]."\n";

    if ($html == 1)
    {
	$htmlBuf = $htmlTitleH.$title.$htmlTitleT;
    }
    if ($text == 1)
    {				
	$textBuf = $title."\n";
    }
  
  getData:
    while ($nextLine = <DATA>)
    {
	last getData if ($nextLine eq "\n");
	    
	@data = split (/\t/, $nextLine);
	chop($data[1]);
	
	$docTitle = $data[1];
	if ($htmlTitles == 1)
	{
	    if ($data[1] =~ /\.htm[l]?$/i)
	    {
			$docTitle = &ExtractTitle($rootPath, $data[1]);
	    }
	}
	if ($html == 1)
	{
	    # link from the given $docURL
	    $htmlBuf .= sprintf ("<B>%6d    </B><A HREF=\"%s%s\">%s\n</A>",
				     $data[0], $docURL, $data[1], $docTitle);
	  
	}
	if ($text == 1)
	{
	  
	    $textBuf .= sprintf ("%6d    %s\n", $data[0], $docTitle);
	}
    }
    if (($nextLine = <DATA>) ne "\n")
    {
	# comment after the list ... 
	@data = split (/\t/, $nextLine);
	$buf = sprintf ("          $data[0]");
	$htmlBuf .= $buf;
	$textBuf .= $buf;
       
    }
    if ($html == 1)
    {
	$htmlBuf .= $htmlEndReport;
    }
    if ($text == 1)
    {
	$textBuf .= $textLine;
    }
    &Output ($htmlBuf, $textBuf);
}

sub MatchList
{
    local ($title, @data);
    local ($textBuf, $htmlBuf, $buf, $totalTextBuf, $totalHtmlBuf);
    local ($dataTextBuf, $dataHtmlBuf);

    $title = <DATA>; 
    if ($html == 1)
    {
	$htmlBuf = "<CENTER>\n<P><FONT SIZE = +1>$title<BR>";
	
    }
    if ($text == 1)
    {				
	$textBuf = $title;
    }

    #get match pattern list
  getMatch:
    while ($nextLine = <DATA>)
    {
	last getMatch if ($nextLine eq "\n");
	if ($html == 1)
	{
	    $htmlBuf .= "<B>$nextLine</B><BR>";
	}
	if ($text == 1)
	{
	    $textBuf .= $nextLine;
	}
    }

    $title = <DATA>;
    if ($html == 1)
    {
	$htmlBuf .= sprintf("$title</FONT><P></CENTER><PRE>\n");
    }
    if ($text == 1)
    {		       
	$textBuf .= "$title\n";
    }
	
  getData:
    while ($nextLine = <DATA>)
    {
	last getData if ($nextLine eq "\n");
	    
	@data = split (/\t/, $nextLine);
	
	if ($html == 1)
	{
	    $dataHtmlBuf .= sprintf ("<B>%6d    </B>%s", $data[0], $data[1]);
	}
	if ($text == 1)
	{
	    $dataTextBuf .= sprintf ("%6d    %s", $data[0], $data[1]);
	}
	
    }
    if (($nextLine = <DATA>) ne "\n")
    {
	# comment after the list ... 
	@data = split (/\t/, $nextLine);
	if ($#data > 0)
	{
	    $totalTextBuf .= sprintf ("$data[0] $data[1] $data[2] $data[3] $data[4]\n");
	    $totalHtmlBuf .= sprintf ("$data[0] <B>$data[1]</B> $data[2] <B>$data[3]</B> $data[4]\n");
	}
	else
        {
	    $totalTextBuf = sprintf ("          $data[0]");
	    $totalHtmlBuf = $totalTextBuf;
	  
	}
    }

    if ($html == 1)
    {
	$htmlBuf .= $totalHtmlBuf;
	$htmlBuf .= $dataHtmlBuf;
	$htmlBuf .= $htmlEndReport;
    }
    if ($text == 1)
    {
	$textBuf .= $totalTextBuf;
	$textBuf .= $dataTextBuf;
	$textBuf .= $textLine;
    }
    &Output ($htmlBuf, $textBuf);
}

sub MatchClientsList
{
    local ($title, @data);
    local ($textBuf, $htmlBuf, $buf, $totalTextBuf, $totalHtmlBuf);
    local ($dataTextBuf, $dataHtmlBuf);

    $title = <DATA>; 
    if ($html == 1)
    {
	$htmlBuf = "<CENTER>\n<P><FONT SIZE = +1>$title<BR>";
	
    }
    if ($text == 1)
    {				
	$textBuf = $title;
    }

    #get match pattern list
  getMatch:
    while ($nextLine = <DATA>)
    {
	last getMatch if ($nextLine eq "\n");
	if ($html == 1)
	{
	    $htmlBuf .= "<B>$nextLine</B><BR>";
	}
	if ($text == 1)
	{
	    $textBuf .= $nextLine;
	}
    }

    $title = <DATA>;
    if ($html == 1)
    {
	$htmlBuf .= sprintf("$title</FONT><P></CENTER><PRE>\n");
    }
    if ($text == 1)
    {		       
	$textBuf .= "$title\n";
    }
	
  getData:
    while ($nextLine = <DATA>)
    {
	last getData if ($nextLine eq "\n");
	    
	@data = split (/\t/, $nextLine);
	chop($data[1]);
	
	$docTitle = $data[1];
	if ($htmlTitles == 1)
	{
	    if ($data[1] =~ /.html?$/i)
	    {
		$docTitle = &ExtractTitle($rootPath, $data[1]);
	    }
	}
	if ($html == 1)
	{
	    $dataHtmlBuf .= sprintf ("<B>%6d    </B><A HREF=\"%s%s\">%s\n</A>",$data[0], $docURL,
$data[1], $docTitle);
	}
	if ($text == 1)
	{
	    $dataTextBuf .= sprintf ("%6d    %s\n", $data[0], $docTitle);
	}
    }
    if (($nextLine = <DATA>) ne "\n")
    {
	# comment after the list ... 
	@data = split (/\t/, $nextLine);
	if ($#data > 0)
	{
	    $totalTextBuf .= sprintf ("\n$data[0] $data[1] $data[2] $data[3] $data[4]\n");
	    $totalHtmlBuf .= sprintf ("\n$data[0] <B>$data[1]</B> $data[2] <B>$data[3]</B> $data[4]\n");
	}
	else
        {
	    $totalTextBuf = sprintf ("\n          $data[0]");
	    $totalHtmlBuf = $totalTextBuf;
	  
	}
    }

    if ($html == 1)
    {
	$htmlBuf .= $totalHtmlBuf;
	$htmlBuf .= $dataHtmlBuf;
	$htmlBuf .= $htmlEndReport;
    }
    if ($text == 1)
    {
	$textBuf .= $totalTextBuf;
	$textBuf .= $dataTextBuf;
	$textBuf .= $textLine;
    }
    &Output ($htmlBuf, $textBuf);
}


sub XReport
{
    local ($title, @data);
    local ($textBuf, $htmlBuf, $buf);

    $textBuf = $htmlBuf = "";

    $title = <DATA>;
    if ($html == 1)
    {
	$htmlBuf = $htmlTitleH.$title.$htmlTitleT;
    }
    if ($text == 1)
    {				
	$textBuf = $title."\n";
    }

    $title = <DATA>;
    @data = split (/\t/,$title);
    $textBuf .= sprintf("%6s    %s\n", $data[0], $data[1]);
    $htmlBuf .= sprintf("%6s    %s\n", $data[0], $data[1]);
  
  getData:
    while ($nextLine = <DATA>)
    {
	last getData if ($nextLine eq "\n");
	    
	@data = split (/\t/, $nextLine);
	
	if ($html == 1)
	{
	    $htmlBuf .= sprintf ("<B>%6d    </B>%s", $data[0], $data[1]);
	}
	if ($text == 1)
	{
	    $textBuf .= sprintf ("%6d    %s", $data[0], $data[1]);
	}
	
    }

    if (($nextLine = <DATA>) ne "\n")
    {
	# comment after the list ... 
	@data = split (/\t/, $nextLine);
	if ($html == 1)
	{
	    $htmlBuf .= "\n$data[0] <B>$data[1]</B> $data[2]";
	}
	if ($text == 1)
	{
	    $textBuf .= "\n$data[0] $data[1] $data[2]";;
	}
    }

    if ($html == 1)
    {
	$htmlBuf .= $htmlEndReport;
    }
    if ($text == 1)
    {
	$textBuf .= $textLine;
    }

    &Output ($htmlBuf, $textBuf);
	
}

sub Status
{
    local ($title, @data);
    local ($textBuf, $htmlBuf, $buf);

    $textBuf = $htmlBuf = "";

    $title = <DATA>;
    if ($html == 1)
    {
	$htmlBuf = $htmlTitleH.$title.$htmlTitleT;
    }
    if ($text == 1)
    {				
	$textBuf = $title."\n";
    }
  
  getData:
    while ($nextLine = <DATA>)
    {
	last getData if ($nextLine eq "\n");
	    
	@data = split (/\t/, $nextLine);
	
	if ($html == 1)
	{
	    $htmlBuf .= sprintf ("<B>%6d    </B>%s     %s", $data[0], $data[1],$data[2]);
	}
	if ($text == 1)
	{
	    $textBuf .= sprintf ("%6d    %s     %s", $data[0], $data[1], $data[2]);
	}
	
    }
    if (($nextLine = <DATA>) ne "\n")
    {
	# comment after the list ... 
	@data = split (/\t/, $nextLine);
	$buf = sprintf ("          $data[0]");
	$htmlBuf .= $buf;
	$textBuf .= $buf;
       
    }
    if ($html == 1)
    {
	$htmlBuf .= $htmlEndReport;
    }
    if ($text == 1)
    {
	$textBuf .= $textLine;
    }
    &Output ($htmlBuf, $textBuf);

}


sub Browsers
{
    local ($chartSet) = @_;
    local ($title, @data, $gotData);
    local ($textBuf, $htmlBuf, $buf);

    $textBuf = $htmlBuf = "";

    $title = <DATA>;
    if ($html == 1)
    {
	$htmlBuf = $htmlTitleH.$title.$htmlTitleT;
    }
    if ($text == 1)
    {				
	$textBuf = $title."\n";
    }

    $gotData = 0;
  getData:
    while ($nextLine = <DATA>)
    {
	last getData if ($nextLine eq "\n");
	    
	@data = split (/\t/, $nextLine);
	
	if ($html == 1)
	{
	    $htmlBuf .= sprintf ("<B>%6s%s    </B>%s", $data[0], "%", $data[1]);
	}
	if ($text == 1)
	{
	    $textBuf .= sprintf ("%6s%s    %s", $data[0], "%", $data[1]);
	}
	$gotData = 1;
    }
    if (($nextLine = <DATA>) ne "\n")
    {
	# comment after the list ... 
	@data = split (/\t/, $nextLine);
	$buf = sprintf ("          $data[0]");
	$htmlBuf .= $buf;
	$textBuf .= $buf;
       
    }
    if ($html == 1)
    {
	if ($gotData == 1)
	{
	    $htmlBuf .= "\n<B>Chart:</B> <A HREF=\"$chartURL/$chartSet\">";
	    $htmlBuf .= "<IMG BORDER=0 SRC=\"$chartImgURL"."all.gif\"";
	    $htmlBuf .= " ALT = Chart ALIGN = MIDDLE></A>";
	}
	$htmlBuf .= $htmlEndReport;
    }
    if ($text == 1)
    {
	$textBuf .= $textLine;
    }
    &Output ($htmlBuf, $textBuf);

}

sub Process
{
    local (@data);
    local ($textBuf, $htmlBuf, $buf);

    $textBuf = $htmlBuf = "";

    @data = split( /\t/, <DATA>);
    
    $data[1] =~ s/,/, /g;

    if ($html == 1)
    {
	$htmlBuf = "<P><B>$data[0] $data[1]</B><p><PRE>";
    }
    if ($text == 1)
    {				
	$textBuf = "$data[0] $data[1]\n";
    }
    
    @data = split( /\t/, <DATA>);
    if ($html == 1)
    {
	$htmlBuf .= sprintf("%-22s%-10s%-10s%-10s%-10s%-10s%-11s%-10s\n", 
		       $data[0], $data[1], $data[2], 
		       $data[3], $data[4], $data[5], $data[6], $data[7]);
    }
    if ($text == 1)
    {
	$textBuf .= sprintf("%s\n%-10s%-10s%-10s%-10s%-10s%-11s%-10s\n", 
		       $data[0], $data[1], $data[2], 
		       $data[3], $data[4], $data[5], $data[6], $data[7]);
    }
 
  getData:
    while ($nextLine = <DATA>)
    {
	last getData if ($nextLine eq "\n");
	    
	@data = split (/\t/, $nextLine);
	if ($html == 1)
	{
	    $htmlBuf .= sprintf( "%-22s%-10s%-10s%-10d%-10d%-10d%-11d%-d\n",
			     $data[0], $data[1], $data[2], 
			     $data[3], $data[4], $data[5], $data[6], $data[7]);
	}
	if ($text == 1)
	{
	    $textBuf .= sprintf("%s\n%-10s%-10s%-10s%-10s%-10s%-11s%-10s\n", 
			     $data[0], $data[1], $data[2], 
			     $data[3], $data[4], $data[5], $data[6], $data[7]);
	}
       
    }
    if ($html == 1)
    {
	$htmlBuf .= $htmlEndReport;
    }
    if ($text == 1)
    {
	$textBuf .= $textLine;
    }
    &Output ($htmlBuf, $textBuf);
}

sub ReportTitle
{
    local ($title, @data);
    local ($textBuf, $htmlBuf, $buf);

    $textBuf = $htmlBuf = "";

    $title = <DATA>;
    @data = split (/\t/,$title);
    if ($#data > 1)
    {
	if ($html == 1)
	{
	    $htmlBuf = "<FONT SIZE = +2><CENTER>$data[0] ";
	    $htmlBuf .= "&nbsp; <B>$data[1]</B>";
	    $htmlBuf .= "&nbsp; $data[2] &nbsp; <B>$data[3]</B>";
	    $htmlBuf .= "</FONT></CENTER><P><HR><P>\n";
      
	}
	if ( $text == 1)
	{
	    $textBuf = "$data[0]  $data[1]  $data[2]  $data[3]\n";
	}
    }
    elsif ($#data == 1)
    {
	if ($html == 1)
	{
	    $htmlBuf = "<FONT SIZE = +2><CENTER>$data[0] ";
	    $htmlBuf .= "&nbsp;<B>$data[1]</B>";
	    $htmlBuf .= "</FONT></CENTER><P><HR><P>\n";
	  
	}
	if ($text == 1)
	{
	    $textBuf = "$data[0]  $data[1]\n";
	}
    }
    else
    {
	#error string 
	if ($html == 1)
	{
	    $htmlBuf = "<FONT SIZE = +2><CENTER>";
	    $htmlBuf .= "$data[0]";
	    $htmlBuf .= "</FONT></CENTER><P><HR><P>\n";
	  
	}
	if ($text == 1)
	{
	    $textBuf = "$data[0]\n";
	}
    }

    # save this for printing at the end 
    @reg = split (/\t/, <DATA>);

    &Output ($htmlBuf, $textBuf.$textLine);
}
  
sub DailyTransfer
{
    local ($title, @data);
    local ($textBuf, $htmlBuf, $buf);

    $textBuf = $htmlBuf = "";

    $title = <DATA>;
    if ($html == 1)
    {
	$htmlBuf = $htmlTitleH.$title.$htmlTitleT;
    }
    if ($text == 1)
    {				
	$textBuf = $title."\n";
    }
    
    @data = split( /\t/, <DATA>);
    
    $buf = sprintf("%-20s%12s%12s%13s\n", 
		      $data[0], $data[1], $data[2], $data[3]);
    

  getData:
    while ($nextLine = <DATA>)
    {
	last getData if ($nextLine eq "\n");
	    
	@data = split (/\t/, $nextLine);
	$buf .= sprintf( "%-20s%12.2f%12.2f%12d\n",
			  $data[0], $data[1], $data[2], $data[3]);
       
    }
    if ($html == 1)
    {
	$htmlBuf .= $buf;
	$htmlBuf .= "\n";

	$htmlBuf .= "<B>Chart:</B> <A HREF=\"$chartURL/tdtall.bl\">";
	$htmlBuf .= "<IMG BORDER=0 SRC=\"$chartImgURL"."all.gif\"";
	$htmlBuf .= " ALT = Chart All ALIGN = MIDDLE></A>";

	
	$htmlBuf .= "<br>";
	
	$htmlBuf .= $htmlEndReport;
    }
    if ($text == 1)
    {
	$textBuf .= $buf.$textLine;
    }
    &Output ($htmlBuf, $textBuf);
}

sub HourlyTransfer
{
    local ($title, @data);
    local ($textBuf, $htmlBuf, $buf);

    $textBuf = $htmlBuf = "";

    $title = <DATA>;
    if ($html == 1)
    {
	$htmlBuf = $htmlTitleH.$title.$htmlTitleT;
    }
    if ($text == 1)
    {				
	$textBuf = $title."\n";
    }
    
    @data = split( /\t/, <DATA>);
    $buf = sprintf("%-20s%12s%15s\n", 
		      $data[0], $data[1], $data[2]);
 
  getData:
    while ($nextLine = <DATA>)
    {
	last getData if ($nextLine eq "\n");
	    
	@data = split (/\t/, $nextLine);
	$buf .= sprintf( "%-20s%12.2f%12.1f %s\n",
			  $data[0], $data[1], $data[2], "%");
       
    }

    if ($html == 1)
    {
	$htmlBuf .= $buf;
	$htmlBuf .= "\n";

	$htmlBuf .= "<b>Chart:</b> <A HREF=\"$chartURL/thaall.bl\">";
	$htmlBuf .= "<IMG BORDER=0 SRC=\"$chartImgURL"."hours.gif\"";
	$htmlBuf .= " ALT = Chart Hours ALIGN = MIDDLE></A>&nbsp;";

	$htmlBuf .= $htmlEndReport;
    }
    if ($text == 1)
    {
	$textBuf .= $buf.$textLine;
    }
    &Output ($htmlBuf, $textBuf);
}
  
sub Output
{
    local ($htmlBuf, $textBuf) = @_;

    if ($text == 1 && $textBuf ne "")
    {
	if ($email == 1)
	{
            print EMAIL $textBuf;
	}
	if ($screen == 1)
	{
	    print $textBuf;
	}
    }
    if ($html == 1 && $htmlBuf ne "")
    {
	print OUT $htmlBuf;
    }
}

sub PostTextError
{
    local ($errorMsg, $sysError) = @_;

    print "Sorry, SurfReport Found an Error:\n";
    print "$errorMsg\n";
    if( $sysError ne '')
    {
	print "The system reported this error:\n";
	print "$sysError\n";
    }
    print "Please correct and try again!\n";
    print "\n";
    
}

sub Error
{
    
    local ($errorMsg, $sysError) = @_;
    if ($standAlone == 1)
    {
	 &PostTextError($errorMsg, $sysError);
    }
    else
    {	
    	# 2.07 align behavior to srengine
    	# creates an error file to be picked up by srcheck.cgi
    	$errFile = $settingsFile. ".err";
    	if (open (ERR, 	">$errFile"))
    	{
	    chmod (0666, $errFile);
    	    print ERR "in srreport.cgi:\n";
    	    print ERR $errorMsg, "\n";
	    if (defined ($sysError))
	    {
		print ERR "The system also reported this error: ", $sysError, "\n";
	    }
    		
	    close (ERR);
    	} 
    }
}
    
sub GetSettings
{
    local (@pairs, $buffer, $name, $value, $len);
 
    if (!open (SETTINGS, "$settingsFile"))
    {
		&Error ("Can't open the settings file '$settingsFile'", $!);
		exit;
    }
    seek (SETTINGS, 0, 2);
    $len = tell( SETTINGS);
    seek (SETTINGS, 0, 0);
    read(SETTINGS, $buffer, $len);
    @pairs = split( /&/, $buffer);

    foreach $pair (@pairs)
    {
	($name, $value) = split(/=/, $pair);
	$Settings{$name} = $value;
    }

    close (SETTINGS);
    
}

# ExtractTitle
# return the title of an html doc if:
#	the doc can be reached thru the $rootPath
#	the doc has both opening and closing tags
# If title cannot be extracted, return the file name
#
sub ExtractTitle 
{
    local ($rootPath, $fileName) = @_; # get local parameters
    local ($title, $sameLine, $line, $gotTag, $gotAll, $fullName);


    $fullName = $rootPath . $fileName;
    if ( ! open (HTML_FILE, $fullName))
    {
	return $fileName;
    }

    $title = "";
    $gotTag = 0;
    $gotAll = 0;
    while ($line = <HTML_FILE>) 
    {
	$sameLine = 0;
	if ($line =~ /<title\s*>/i) 
	{
	    # get opening tag
	    $title = $';
	    $line = $title;
	    $sameLine = 1;
	    $gotTag = 1;
	}
	if ($gotTag == 1)
	{
	    if ($line =~ /<\/title\s*>/i)
	    {
		# get closing tag
		if ($sameLine == 1)
		{
		    # replace title if both tags on the same line
		    $title = $`;
		}
		else
		{
		    # or just add to it
		    $title .= $`;
		}
		$gotAll = 1;
		last;
	    }
	    elsif ($sameLine == 0) 
	    {
		# multiline titles
		$title .= $line;
	    }
	}
	
    }			
    close (HTML_FILE);		
    
    if ($gotAll == 0)
    {
	return $fileName;
    }
    else
    {
	# to fit in the reports, 
	# scratch the heading & trailing spaces 
	# replace embedded newlines with a single space
	$title =~ s/^\s*//;
	$title =~ s/\s*$//;
	$title =~ s/\s*\n+\s*/ /g;
	if ($title !~ /\S+/)
	{
	    # if title does not have a least one non-space char 
	    return $fileName;
	}
	else
	{
	    return $title;
	}
	
    }
}


#### Rick Debug 5/30/96

sub ResolveIP
{
    local ($ip) = @_;
    local ($addNum);

    $AF_INET = 2;

    $addNum = pack( 'C4', split(/\./, $ip));

    (gethostbyaddr( $addNum, $AF_INET))[0];
}



