#!/usr/bin/perl

#########################################################
#  surfreport.cgi
#  dispatcher for all other cgi scripts and programs
#  used with SurfReport
#
#  V2.1.7 Single User 
#  Author: Michel G Delory
#  Created: Sep 16 1996
#  Last Rev: Dec 12 1997
#
#  execute SR applications and scripts depending on the PATH_INFO variable
#  nothing->configure
#  /launch->receives form params on stdin, launch surfreport
#  /check/temp_file ->monitors SR progress, catch errors and display them
#  /chart/format_file ->start charting app with given format file
#  /design/format_file ->launch chart design script->design form
#  /chart ->launch chart app with stdin params form the design form
#  /validate/temp_file ->process validation form
#  /menu -> gives a set of URLs to more dispatcher functions
#
#############################################################

$httpHeader = "Content-type: text/html\n\n";
$errorHeader = "<HTML><HEAD><TITLE>SurfReport Error</TITLE></HEAD>";
$messageHeader = "<HTML><HEAD><TITLE>SurfReport Message</TITLE></HEAD>";

#######################
# ENVIRONMENT VARIABLES 
#######################

if (defined ($ARGV[0]))
{    
    $standAlone = 1;
     $inputFile = $ARGV[0];
}
else
{    
    $standAlone = 0;

    #get path info dispatcher commands
    #strip the optional leading '/'	
    $command = $ENV{PATH_INFO};

	#patch for microsoft sp3 for NT 4.0
	if (($pos = index ($command, "surfreport.cgi")) >= $[)
	{
		$command = substr ($command, $pos + length ("surfreport.cgi"));
		$ENV{PATH_INFO} = $command;
	}

    $command =~ s/^\///;	
    @commands = split ('\/', $command);

}

# install time variables
$SURFREPORT_HOME="d:/netscape/suitespot/htdocs/surfreport/";
$PERL_HOME="D:/WINNT/System32/PERL.EXE";
$MAIL_SERVICE="D:/WINNT/System32/BLAT.EXE";
$PLATFORM="OS_NT";
 
# run time variables
if (($err = &GetProperties()) ne "")
{
    print $httpHeader;	    
    print $errorHeader;	    
    &FatalError($err);	    
    exit (-1);	
}


# if your isp kills off lengthy background processes
# uncomment the following line
# you will not get a notice page and
# your browser will wait for the report to be finished
# you may be subject to the web server cutting you off if
# the process is taking longer than the time limit set for
# CGI processes
#$SURFREPORT_DIRECTOUTPUT = 1;


if ($standAlone == 0)
{    
    #url pointing to this dispatcher    
    #automatically set at run time    
    $serverName = &GetEnv("SERVER_NAME", 1);
    $serverPort = &GetEnv("SERVER_PORT", 1);
	
    # wipe any blanks left courtesy of Netscape Servers    
    $serverName =~ s/\s*$//;    
    $serverPort =~ s/\s*$//;    

    $this = "http://";    
    $this .= $serverName;

    # include non standard http port in the dispatcher URL
    if ($serverPort ne "" && $serverPort ne "80")
    {
	$this .= ":" . $serverPort;
    }

    $scriptName = &GetEnv ("SCRIPT_NAME", 1);
    $scriptName =~ s/\s*$//;

    if ($scriptName !~ /^\//)
    {
	# sometimes, the leading slash is not included in SCRIPT_NAME
	$this .= "/";
    }
	    
    $this .= $scriptName;    
}
else
{		
    # running standalone requires the correct URL to the dispatcher
    # to be set in the srconf.cgi file
    # this is done during configuration
    # but you must update it if your path changes		    
    $this = "";
}

# some servers may think it wise to stick 
# a slash at the end of scriptname
# get rid of it
$this =~ s/\/$//;
$DISPATCHER_URL = $this;


#################### PROCESSING ####################

#set env vars for child processes
&SetEnv();
# get the path of the program to be executed
$programName = $SURFREPORT_HOME;
$programName .= "cgi/";

if ($standAlone == 1)
{    
	
    # can only get a text report when running    
    # stand alone    
    $programName .= "sr.cgi";    
    @commandLine = ($PERL_HOME, $programName, $inputFile);    
}
else
{  
    # switch based on path info    
    if ($command eq "")   
    {	
	$programName .= "srconfig.cgi";	
	@commandLine = ($PERL_HOME, $programName);    
    }    
    else    
    {	
	if ($commands[0] eq "launch")	
	{	
	   
	    $programName .= 'sr.cgi';	    
	    @commandLine = ($PERL_HOME, $programName);	
	    
	}	
	elsif ($commands[0] eq "check")
	{		    
	    $programName .= "srcheck.cgi";	    
	    @commandLine = ($PERL_HOME, 			    
			    $programName, 			    
			    $commands[1], #nameBase 			    
			    $commands[2], #tmp file base
			    $commands[3]); #count
	
	}	
	elsif ($commands[0] eq "chart")	
	{	    
	    
	    if ($PLATFORM eq "OS_NT")
	    {
		$programName .= "srchart.exe";
	    }
	    else
	    {
		$programName .= "srchart.cgi";
	    }
	    if ($#commands == $[)	    
	    {			
		@commandLine = ($programName);	    
	    }	    
	    else	    
	    {		
		@commandLine = ($programName, 				
				$commands[1], #nameBase
				$commands[2]); #set (.bl) file	    

	    }	
	}	
	elsif ($commands[0] eq "design")	
	{	    
	    $programName .= "srdesign.cgi";	    
	    @commandLine = ($PERL_HOME, $programName, 			    
			    $commands[1], #nameBase			    
			    $commands[2]); #set (.bl) file	
	}	
	elsif ($commands[0] eq "config")	
	{	    
	    $programName .= "srconfig.cgi";	    
	    
	    #arguments are kept in PATH_INFO	    
	    @commandLine = ($PERL_HOME, $programName);	
	}	
	elsif ($commands[0] eq "validate")	
	{	   
	    if ($PLATFORM eq "OS_NT")
	    {	
		$programName .= "srvalidate.exe";	
	    }
	    else
	    {
		$programName .= "srvalidate.cgi";	 
	    }   
			
	    #arguments are kept in PATH_INFO	    
	    @commandLine = ($programName);	
	}
	elsif ($commands[0] eq "debug")
	{
	    &Debug();
	    exit (-1);
	}
	elsif ($commands[0] eq "env")
	{
	    &Env();
	    exit (-1);
	}
	elsif ($commands[0] eq "menu")
	{
	    &Menu();
	    exit (-1);
	}
	elsif ($commands[0] eq "reg")
	{
	    if ($PLATFORM eq "OS_NT")
	    {	
		$programName .= "srengine.exe";	
	    }
	    else
	    {
		$programName .= "srengine.cgi";	 
	    }   
			
	    #no arguments will return reg code & user name  
	    @commandLine = ($programName);	
	}
	else	
	{	    
	    #unknown command	    
	    $err = "Unknown command: ";	    
	    $err .= $ENV{PATH_INFO};	    
	    print $httpHeader;	    
	    print $errorHeader;	    
	    &FatalError($err);	    
	    exit (-1);	
	}    			    
    }
}

if (!exec @commandLine)
{    
    #if exec returned, there's been an error    
    #print an error message if it returns, but very unlikely to do so    
    #most likely event is a server error returned to the user    
    print $httpHeader;    
    print $errorHeader;    
    &FatalError ("Can't execute '$programName'\n$!");
}

################### SUBROUTINES ###################

#
# set environment variables for child processes
#
sub SetEnv
{        
    $ENV{'PLATFORM'} = $PLATFORM;
    $ENV{'SURFREPORT_HOME'} = $SURFREPORT_HOME;   
    $ENV{'SURFREPORT_WEBHOME'} = $SURFREPORT_WEBHOME;  
    $ENV{'SURFREPORT_DOCROOT'} = $SURFREPORT_DOCROOT;  
    $ENV{'SURFREPORT_OUT'} = $SURFREPORT_OUT;    
    $ENV{'SURFREPORT_CONFIG'} = $SURFREPORT_CONFIG;    
    $ENV{'SURFREPORT_LIB'} = $SURFREPORT_LIB;
    $ENV{'PERL_HOME'} = $PERL_HOME;    
    $ENV{'MAIL_SERVICE'} = $MAIL_SERVICE;    
    $ENV{'COMPRESS_HOME'} = $COMPRESS_HOME;
    $ENV{'DISPATCHER_URL'} = $DISPATCHER_URL;   
	
    if ($SURFREPORT_MASTERFILTER ne "")
    {
	$ENV{'SURFREPORT_MASTERFILTER'} = $SURFREPORT_MASTERFILTER;
    }

    if ($standAlone == 1)    
    {	
    	$ENV{'STAND_ALONE'} = 'YES';
    }
    else
    {
    	if (defined ($SURFREPORT_DIRECTOUTPUT))
        {
            $ENV{'SURFREPORT_DIRECTOUTPUT'} = $SURFREPORT_DIRECTOUTPUT;
        }
		 
    }
    
    $ENV{'SURFREPORT_MULTIUSER'} = "NO";

}

#
# print surfreport environment
#
sub Debug
{    
	local ($err);     
	print $httpHeader;    
	print $messageHeader;

	$err = "<pre>\n";	
	$err .= "SURFREPORT_HOME: >$SURFREPORT_HOME<\n";   
	$err .= "SURFREPORT_WEBHOME: >$SURFREPORT_WEBHOME<\n"; 
	$err .= "SURFREPORT_DOCROOT: >$SURFREPORT_DOCROOT<\n";    
	$err .=  "SURFREPORT_OUT: >$SURFREPORT_OUT<\n";    
	$err .= "SURFREPORT_CONFIG: >$SURFREPORT_CONFIG<\n";   
	$err .= "SURFREPORT_LIB: >$SURFREPORT_LIB<\n";
	$err .= "PERL_HOME: >$PERL_HOME<\n";    
	$err .= "PLATFORM: >$PLATFORM<\n";    
	$err .= "MAIL_SERVICE: >$MAIL_SERVICE<\n";    
	$err .= "COMPRESS_HOME: >$COMPRESS_HOME<\n";    
	$err .= "DISPATCHER_URL: >$DISPATCHER_URL<\n";    
	$err .= "SURFREPORT_MASTERFILTER: >$SURFREPORT_MASTERFILTER<\n";
	
	if (defined ($SURFREPORT_DIRECTOUTPUT))
	{
	    $err .= "SURFREPORT_DIRECTOUTPUT: >$SURFREPORT_DIRECTOUTPUT<\n";
	}
    
	if ($PLATFORM eq "OS_UNIX")
	{
	    $err .= "\n";
	    $data = `pwd`;
	    chop $data;
	    $err .= "Current directory for this machine: >$data<\n";
	    
	    $data = `uname -a`;
	    chop $data;
	    $err .= "Platform type and processor: >$data<\n";
	}

	$err .= "</pre>";

	&Message ("Dispatcher Settings", $err);
}

#
# print web environment (includes surfreport environment)
#
sub Env
{    
    local ($err);
	     
    print $httpHeader;    
    print $messageHeader;
    
    $err = "<pre>\n";
    foreach $env (sort keys (%ENV))
    {
	$err .= $env . " = " . $ENV{$env} . "\n";
    }
    $err .= "</pre>";
   
    
    &Message("Web Environment", $err);
}

#
# display an error message
#
sub FatalError
{    
	local ($err) = @_;
	
	print "<body BGCOLOR = #FFFFFF TEXT = #OOOOOO >\n";
	print "<H3>Dispatcher Error</H3>\n";        
	print "<pre>\n\n";    
	print $err;    
	print "\n\n</pre>";    
	&PrintHTMLEnd();
}
#
# display a message
#
sub Message
{    
	local ($title, $msg) = @_;
	
	print "<body BGCOLOR = #FFFFFF TEXT = #OOOOOO >\n";
	print "<H3>$title</H3>\n";        
	print $msg;    
	print "&nbsp;<P>";    

	&PrintHTMLEnd();
}

#
# print end of page credits 
#
sub PrintHTMLEnd
{    
    print "<H5>SurfReport 2.1.7<br>Copyright &copy; 1996, 1997 ";
    print "<A HREF = \"http://www.bienlogic.com\">Bien Logic, Inc.</A></H5>";
    print "</BODY></HTML>";
}

#
# get an env variable, exit with error if not found
#
sub GetEnv
{	
	local ($name, $dieOnError) = @_;
	local ($env);
	
	$env = $ENV{$name};	
	if ((!defined ($env) || $env !~ /\S+/) && $dieOnError != 0)	
	{		
		# missing or blank env var.		
		# lets get out of here real quick			
		$err = "Environment variable $name is missing or blank!\n";		
		$err .= "Your Web server may not be properly configured.\n";		
		$err .= "Contact your server administrator.\n";			
		
		print $httpHeader;    	
		print $errorHeader;    	
		&FatalError ($err);    	
		exit (-1);    
	}         
	return $env;
}


#
# print menu urls
#
sub Menu
{
    local ($msg);
	     
    print $httpHeader;    
    print $messageHeader;
    
    $msg = "<UL>\n";
    $msg .= "<LI><A HREF = \"$DISPATCHER_URL/config\">Change SurfReport's configuration</A></LI>\n";
    $msg .= "<LI><A HREF = \"$DISPATCHER_URL/config/form\">Change defaults in your interface form</A></LI>\n";
    $msg .= "<LI><A HREF = \"$DISPATCHER_URL/debug\">Display SurfReport's settings</A></LI>\n";
    $msg .= "<LI><A HREF = \"$DISPATCHER_URL/env\">Display your current Web environment</A></LI>\n";
    $msg .= "<LI><A HREF = \"$DISPATCHER_URL/reg\">Display your registration status</A></LI>\n";

    $msg .= "</UL>";

    require $SURFREPORT_CONFIG;

    if ($configured == 1)
    {
	$msg .= "<P>&nbsp;<P>";
	$msg .= "[<A HREF=\"$surfHtmlURL\">Back to SurfReport home page</A>]";
    }

    &Message("Dispatcher Menu", $msg);
}


#
# get user properties stored in a config file
#
sub GetProperties
{
    local ($propertiesPath, $foundVars);
    local (%vars, $msg, $key, $val);

    # open the user list
    $propertiesPath = $SURFREPORT_HOME. "config/properties.cgi";

    if (open (PROPERTIES, $propertiesPath))
    {
	
	# run thru the list
	$msg = "";
	# get output folder, config file name and master filter
	$foundVars = 0;
      getvars:
	while (<PROPERTIES>)
	{
	    if (/^\s*#/)
	    {
		next getvars;
	    }

	    if (/^\s*(\w+)\s*=\s*([^\n#]*)/ )
	    {
		$key = $1;
		$val = $2;

		$vars{$key} = $val;
		$foundVars++;
		$msg .= $key . " = " . $val . "\n";
	    }	
	}
    
	close (PROPERTIES);
    }
	
    # need to init environment
    # and set defaults even if the property sheet is not found		
    if ($vars {"web"} =~ /\S+/)
    {
	$SURFREPORT_WEBHOME = $vars{"web"};
    }
    else
    {
	#use main output folder when this is blank
	$SURFREPORT_WEBHOME = $SURFREPORT_HOME;
    }
	
    # make sure paths are slash terminated
    if ($SURFREPORT_WEBHOME !~ /\/$/)
    {
	$SURFREPORT_WEBHOME .= "/";
    }

    if ($vars {"docroot"} =~ /\S+/)
    {
	$SURFREPORT_DOCROOT = $vars{"docroot"};
    }
    else
    {
	#use webhome parent folder when this is blank

	$path = $SURFREPORT_WEBHOME;
	chop ($path);

	if (($pos = rindex ($path, "/")) == $[ - 1)
	{
		if (($pos = rindex ($path, "\\")) == $[ - 1)
		{
			$pos = length ($path) - 1;
		}
	}

	$SURFREPORT_DOCROOT = substr ($path, 0, $pos + 1);

    }
	
    # make sure paths are slash terminated
    if ($SURFREPORT_DOCROOT !~ /\/$/)
    {
	$SURFREPORT_DOCROOT .= "/";
    }


    # surfreport's output folder
    $SURFREPORT_OUT = $SURFREPORT_WEBHOME . "srout/";
    
    if ($vars {"config"} =~ /\S+/)
    {
	$SURFREPORT_CONFIG = $vars{"config"};
    }
    else
    {
	#use main output folder when this is blank
	$SURFREPORT_CONFIG = $SURFREPORT_HOME . "config/srconf.cgi";
    }
	
    if ($vars {"filter"} =~ /\S+/)
    {
	$SURFREPORT_MASTERFILTER = $vars{"filter"};
    }
    
    
    if ($vars {"compress"} =~ /\S+/)
    {
	$COMPRESS_HOME = $vars{"compress"};
    }
    else
    {
	#use main output folder when this is blank
	$COMPRESS_HOME = "";
    }

    #dummy value for user_id
    $SURFREPORT_USERID = 0; 

    #path to master configuration file
    $SURFREPORT_LIB = $SURFREPORT_HOME . "config/srlib.cgi";
    
   return "";
    
}
